<?php

namespace App\Controller;

use App\Entity\AdManager;
use App\Entity\Ayarlar;
use App\Entity\Raporlar;
use App\Entity\Siteler;
use Doctrine\ORM\EntityManagerInterface;
use Google\AdsApi\AdManager\v202311\ReportQueryAdUnitView;
use Google\AdsApi\AdManager\v202408\ReportDownloadOptions;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;
use Google\AdsApi\AdManager\v202408\ReportQuery;
use Google\AdsApi\AdManager\v202408\ReportJob;
use Google\AdsApi\AdManager\v202408\ReportService;
use Google\AdsApi\AdManager\v202408\DateRangeType;
use Google\AdsApi\AdManager\v202408\Dimension;
use Google\AdsApi\AdManager\v202408\Column;
use Google\AdsApi\AdManager\v202408\ExportFormat;
use Google\AdsApi\AdManager\v202408\Date;
use Google\AdsApi\AdManager\Util\v202408\ReportDownloader;
use Google\AdsApi\Common\OAuth2TokenBuilder;
use Google\AdsApi\AdManager\AdManagerSessionBuilder;

#[Route('/ad-manager/', name: 'app_ayarlar_homepage')]
class AdManagerController extends BaseController
{


    // Yarım saat veya 1 saatte 1 kez çalışacak.
    #[Route('', name: 'app_adManager_homepage')]
    public function index(EntityManagerInterface $em): Response
    {
        $this->dilKontrol();

        $adManagerKontrol = $em->getRepository(AdManager::class)->findBy(['status'=>false]);

        if(count($adManagerKontrol)==0) {
            $oAuth2Credential = (new OAuth2TokenBuilder())->fromFile()->build();
            $session = (new AdManagerSessionBuilder())->fromFile('adsapi_php.ini')
                ->withOAuth2Credential($oAuth2Credential)
                ->build();

            $adManagerServices = new \Google\AdsApi\AdManager\AdManagerServices();
            $reportService = $adManagerServices->get($session, ReportService::class);

            // Rapor sorgusu oluştur
            $reportQuery = new ReportQuery();
            $reportQuery->setDimensions([Dimension::SITE_NAME]);
            $reportQuery->setColumns([
                Column::TOTAL_LINE_ITEM_LEVEL_IMPRESSIONS,
                Column::AD_EXCHANGE_LINE_ITEM_LEVEL_CLICKS,
                Column::TOTAL_LINE_ITEM_LEVEL_ALL_REVENUE,
                Column::TOTAL_LINE_ITEM_LEVEL_WITH_CPD_AVERAGE_ECPM,
                Column::TOTAL_ACTIVE_VIEW_VIEWABLE_IMPRESSIONS_RATE
            ]);
            $reportQuery->setAdUnitView(ReportQueryAdUnitView::HIERARCHICAL);
            // Set the start and end dates or choose a dynamic date range type.
            $reportQuery->setDateRangeType(DateRangeType::CUSTOM_DATE);
            $today = new \DateTime();
            $reportQuery->setStartDate(new Date((int)$today->format('Y'), (int)$today->format('m'), (int)$today->format('d')));
            $reportQuery->setEndDate(new Date((int)$today->format('Y'), (int)$today->format('m'), (int)$today->format('d')));

            $reportJob = new ReportJob();
            $reportJob->setReportQuery($reportQuery);

            $reportJob = $reportService->runReportJob($reportJob);


            $adManager = new AdManager();
            $adManager->setJobId($reportJob->getId());
            $adManager->setStatus(false);
            $adManager->setSave(false);
            $adManager->setTarih(new \DateTime('now'));
            $em->persist($adManager);
            $em->flush();
        }

        return new Response('');


    }

    // her dakika çalışacak
    #[Route('status', name: 'app_adManager_status')]
    public function status(EntityManagerInterface $em): Response
    {
        $this->dilKontrol();


        $adManagerKontrol = $em->getRepository(AdManager::class)->findBy(['status'=>false]);

        if(count($adManagerKontrol)>0) {
            $oAuth2Credential = (new OAuth2TokenBuilder())->fromFile()->build();
            $session = (new AdManagerSessionBuilder())->fromFile('adsapi_php.ini')
                ->withOAuth2Credential($oAuth2Credential)
                ->build();

            $adManagerServices = new \Google\AdsApi\AdManager\AdManagerServices();
            $reportService = $adManagerServices->get($session, ReportService::class);

            // Raporun durumunu kontrol et
            $status = $reportService->getReportJobStatus($adManagerKontrol[0]->getJobId());


            if ($status === 'COMPLETED') {

                $options = new ReportDownloadOptions();
                $options->setExportFormat('XML'); // XML formatında indir
                $options->setUseGzipCompression(false); // Sıkıştırma kullanma

                // İndirme URL'sini al
                $downloadUrl = $reportService->getReportDownloadUrlWithOptions($adManagerKontrol[0]->getJobId(), $options);


                // Dosya yolunu ayarla ve raporu indir
                $filePath = 'xml/report_' . date('m-d-Y') . '.xml';
                copy($downloadUrl, $filePath);


                $adManagerKontrol[0]->setStatus(true);
                $em->persist($adManagerKontrol[0]);
                $em->flush();


            }

        }

        return $this->redirect('/ad-manager/save');
    }

    #[Route('save', name: 'app_adManager_save')]
    public function save(EntityManagerInterface $em): Response
    {
        $this->dilKontrol();
        $adManagerKontrol = $em->getRepository(AdManager::class)->findBy(['save'=>false]);
        if(count($adManagerKontrol)>0) {
            $xmlData = file_get_contents('xml/report_' . date('m-d-Y') . ".xml");
            $xml = simplexml_load_string($xmlData, "SimpleXMLElement", LIBXML_NOCDATA);
            $json = json_encode($xml);
            $array = json_decode($json, TRUE);

            $bindings = [];

            if (count($array) == 0) die;

            foreach ($array as $k_a => $v_a) {
                if (array_key_exists('DataSet', $v_a)) {
                    foreach ($v_a["DataSet"]["Row"] as $k_b => $v_b) {
                        $XFP_SITE_NAME = $v_b["Column"][0];
                        $c1 = $v_b["Column"][1];
                        $c2 = $v_b["Column"][2];
                        $c3 = $v_b["Column"][3];
                        $c4 = $v_b["Column"][4];
                        $c5 = $v_b["Column"][5];
                        $createdArray = array(
                            $c1["@attributes"]["name"] => $c1["Val"],
                            $c2["@attributes"]["name"] => $c2["Val"],
                            $c3["@attributes"]["name"] => $c3["Val"],
                            $c4["@attributes"]["name"] => $c4["Val"],
                            $c5["@attributes"]["name"] => $c5["Val"]
                        );
                        $bindings[$XFP_SITE_NAME["Val"]] = $createdArray;
                    }
                }
            }


            $suan = new \DateTime('now');
            $qb = $em->createQueryBuilder();
            $qb->delete(Raporlar::class, 's')
                ->where("s.thisDate='" . $suan->format('Y-m-d') . "'")
                ->getQuery()
                ->execute();



            $groupedData = [];

            echo '<pre>';
            print_r($bindings);


            foreach (array_reverse($bindings) as $domain => $stats) {
                if(strstr($domain,'.')) {
                    $dd = explode('//', $domain);
                    if (count($dd) > 1) {
                        $dd = $dd[1];
                    } else {
                        $dd = $dd[0];
                    }


                    $bolumGelen = explode('.', $dd);
                    $bak = $bolumGelen[1];



                    $varKey = "";
                   foreach ($groupedData as $yDD => $stats2){
                       $bolum = explode('.',$yDD);
                       $bakilcak = $bolum[0];

                       if($bak ==$bakilcak){
                           $varKey = $yDD;

                       }
                   }


                   if($varKey==""){
                       $groupedData[$domain][$domain] = $stats;
                   }else{
                       $groupedData[$varKey][$domain]  =$stats;
                   }


                }
            }




            foreach ($groupedData as $key_e => $val_k) {

                try {



                    $site = $em->getRepository(Siteler::class)->findBy(['url' => $key_e]);


                    if(count($site)>0) {
                        $adxReservationPubCostDeliveredTop = 0;
                        $adxReservationEcpmDeliveredTop = 0;
                        $adxReservationImpressionsDeliveredTop =0;
                        $adxReservationClicksDeliveredTop =0;
                        $activeViewAdxViewableImpressionsRateTop = 0;



                        foreach ($val_k as $val_e){
                            $adxReservationPubCostDelivered = explode('&nbsp;', htmlentities($val_e["totalReservationPubCostWithCpdDelivered"]))[1];
                            $adxReservationEcpmDelivered = explode('&nbsp;', htmlentities($val_e["totalReservationWithCpdEcpmDelivered"]))[1];;




                            $adxReservationPubCostDeliveredTop += $adxReservationPubCostDelivered;
                            $adxReservationEcpmDeliveredTop += $adxReservationEcpmDelivered;
                            $adxReservationImpressionsDeliveredTop += floatval(str_replace(',', '', $val_e["totalReservationImpressionsDelivered"]));
                            $adxReservationClicksDeliveredTop += floatval(str_replace(',', '', $val_e["adxReservationClicksDelivered"]));
                            $activeViewAdxViewableImpressionsRateTop +=floatval(str_replace('%', '', $val_e["activeViewViewableImpressionsRate"]));

                        }




                        $getCurrent_Percent = $site[0]->getKomisyon();

                        $adxReservationPubCostDeliveredTop = ($adxReservationPubCostDeliveredTop - (($getCurrent_Percent*$adxReservationPubCostDeliveredTop) / 100));
                        $adxReservationEcpmDeliveredTop = $adxReservationEcpmDeliveredTop - (($getCurrent_Percent*$adxReservationEcpmDeliveredTop) / 100);


                        $activeViewAdxViewableImpressionsRateTop = $activeViewAdxViewableImpressionsRateTop/count($val_k);



                        $rapor = new Raporlar();
                        $rapor->setUrl($key_e);
                        $rapor->setAdxReservationImpressionsDelivered(number_format($adxReservationImpressionsDeliveredTop,2,'.',''));
                        $rapor->setAdxReservationClicksDelivered(number_format($adxReservationClicksDeliveredTop,2,'.',''));
                        $rapor->setAdxReservationPubCostDelivered(number_format($adxReservationPubCostDeliveredTop,2,'.',''));
                        $rapor->setAdxReservationEcpmDelivered(number_format($adxReservationEcpmDeliveredTop,2,'.',''));
                        $rapor->setActiveViewAdxViewableImpressionsRate(number_format($activeViewAdxViewableImpressionsRateTop,2,'.',''));
                        $rapor->setThisDate(new \DateTime('now'));
                        $rapor->setPercent($getCurrent_Percent == '' ? 0.01 : $getCurrent_Percent);
                        $rapor->setSite($site[0]);
                        $em->persist($rapor);
                        $em->flush();

                    }

                } catch (\Throwable $th) {
                    echo $th;
                }
            }


           $adManagerKontrol[0]->setSave(true);
            $em->persist($adManagerKontrol[0]);
            $em->flush();

            @unlink('xml/report_' . date('m-d-Y') . '.xml');


        }
        return new Response('');

    }



}