<?php

namespace App\Controller;

use App\Entity\BankaHesaplari;
use App\Entity\Bankalar;
use Doctrine\Persistence\ManagerRegistry;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Mailer\MailerInterface;
use Symfony\Component\PasswordHasher\Hasher\UserPasswordHasherInterface;
use Symfony\Component\Routing\Annotation\Route;

#[Route('/tanimlamalar/bankalar')]
class BankalarController extends BaseController
{

    #[Route('', name: 'app_bankalar_homepage')]
    public function index(): Response
    {
        $this->dilKontrol();

        return $this->render('panel/tanimlamalar/bankalar/index.html.twig');
    }

    #[Route('/result', name: 'app_bankalar_result')]
    public function result(ManagerRegistry $doctrine,Request $request): Response
    {
        $this->dilKontrol();


        $em =  $doctrine->getManager();


        $draw = $request->request->get('draw');
        $row= $request->request->get('start');
        $rowperpage= $request->request->get('length');
        $searchValue= $request->request->all('search')['value'];



        $sql="s.remove is null";
        if($searchValue != ''){
            $sql .=" and (s.adi like '%$searchValue%' or s.kisaAdi like '%$searchValue%')";
        }


        $qb = $em->createQueryBuilder();
        $hepsi=$qb->select('s.id')
            ->from(Bankalar::class, 's')
            ->where("s.remove is null")
            ->getQuery()
            ->getScalarResult();
        $totalRecords = count($hepsi);

        $qb = $em->createQueryBuilder();
        $hepsi=$qb->select('s.id')
            ->from(Bankalar::class, 's')
            ->where($sql)
            ->getQuery()
            ->getScalarResult();

        $totalRecordwithFilter = count($hepsi);
        $qb = $em->createQueryBuilder();
        $veriler=$qb->select('s.id,s.adi,s.kisaAdi,s.logo')
            ->from(Bankalar::class, 's')
            ->where($sql)
            ->setFirstResult($row)
            ->setMaxResults($rowperpage)
            ->getQuery()
            ->getScalarResult();

        $data = array();

        foreach ($veriler as $veri){


            $islem = $this->islemler($veri['id']);

            $resim = "";
            if($veri['logo']==''){
                $resim = '<div class="avatar avatar-md me-3"><span class="avatar-initial rounded-circle bg-label-primary">'.mb_substr(explode(' ',$veri['adi'])[0],0,1,'UTF-8').mb_substr(explode(' ',$veri['adi'])[1]??"",0,1,'UTF-8').'</span></div>';
            }else{
                $resim = '<div class="avatar avatar-md me-3">'.$this->imgOlustur($veri['logo'],48,100).'</div>';
            }


            $data[] = array(
                "adi" => '<div class="d-flex align-items-center">'.$resim.'<div>'.$veri['adi'].'</div>',

                "kisaAdi" => $veri['kisaAdi'],
                    "islem" => $islem,
                );

        }

        $response = array(
            "draw" => intval($draw),
            "recordsTotal" => $totalRecords,
            "recordsFiltered" => $totalRecordwithFilter,
            "data" => $data
        );

        return $this->json($response);
    }

    #[Route('/remove', name: 'app_bankalar_remove')]
    public function remove(Request $request,ManagerRegistry $doctrine): Response
    {
        $this->dilKontrol();
        $em =  $doctrine->getManager();

        $id = $request->request->get('id');

        $veri = $em->getRepository(Bankalar::class)->find($id);
        $veri->setRemove(1);
        $em->persist($veri);
        $em->flush();

        return new Response('');
    }

    #[Route('/modal', name: 'app_bankalar_modal')]
    public function modal(Request $request,ManagerRegistry $doctrine): Response
    {
        $this->dilKontrol();
        $em =  $doctrine->getManager();

        $id = $request->request->get('id');

        $veriler=[];
        if($id!="add"){
            $qb = $em->createQueryBuilder();
            $veriler=$qb->select('s.adi,s.kisaAdi,s.logo')
                ->from(Bankalar::class, 's')
                ->where('s.id='.$id)
                ->getQuery()
                ->getScalarResult();
        }



        return $this->render('panel/tanimlamalar/bankalar/modal.html.twig',array('veriler'=>$veriler,'id'=>$id));
    }
    #[Route('/modalSave', name: 'app_bankalar_modalSave')]
    public function modalSave(Request $request,ManagerRegistry $doctrine,UserPasswordHasherInterface $passwordEncoder,MailerInterface $mailer): Response
    {
        $this->dilKontrol();
        $em =  $doctrine->getManager();

        $id = $request->request->get('id');

        $adi = $request->request->get('adi');
        $kisaAdi = $request->request->get('kisaAdi');
        $gatewayUrl = $request->request->get('gatewayUrl');
        $provisionUrl = $request->request->get('provisionUrl');
        $apiUrl = $request->request->get('apiUrl');


        if($id!="add"){
            $data = $em->getRepository(Bankalar::class)->find($id);
        }else{
            $data = new Bankalar();
        }

        if(($_FILES['resim']['name']??"")!=''){
            $logo=$this->seo_entity($adi).'_logo_'.time();
            move_uploaded_file($_FILES['resim']['tmp_name'], "uploads/".$logo.$this->uzantiBul($_FILES['resim']['type']));
            $logo = $this->imageResizer($logo.$this->uzantiBul($_FILES['resim']['type']),$logo);

            $data->setLogo($logo);
        }



        $data->setAdi($adi);
        $data->setKisaAdi($kisaAdi);
        $data->setGatewayUrl($gatewayUrl);
        $data->setProvisionUrl($provisionUrl);
        $data->setApiUrl($apiUrl);

        $em->persist($data);
        $em->flush();


      return new Response('');
    }



}