<?php

namespace App\Controller;


use App\Entity\ESSablonlar;
use App\Entity\Izinler;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\Persistence\ManagerRegistry;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Mailer\MailerInterface;
use Symfony\Component\PasswordHasher\Hasher\UserPasswordHasherInterface;
use Symfony\Component\Routing\Annotation\Route;


#[Route('/ayarlar/es-sablonlar')]
class ESSablonlarController extends BaseController
{

    #[Route('', name: 'app_esSablonlar_homepage')]
    public function index(): Response
    {
        $this->dilKontrol();

        return $this->render('panel/ayarlar/es_sablonlar/index.html.twig');
    }

    #[Route('/result', name: 'app_esSablonlar_result')]
    public function result(EntityManagerInterface $em,Request $request, RequestStack $session): Response
    {
        $this->dilKontrol();




        $draw = $request->request->get('draw');
        $row= $request->request->get('start');
        $rowperpage= $request->request->get('length');
        $searchValue= $request->request->all('search')['value'];



        $sql="e.remove is null";
        if($searchValue != ''){
            $sql .=" and (m.adi like '%$searchValue%' or e.adi like '%$searchValue%')";
        }


        $qb = $em->createQueryBuilder();
        $hepsi=$qb->select('e.id')
            ->from(ESSablonlar::class, 'e')
            ->join('e.modul','m')
            ->where("e.remove is null")
            ->getQuery()
            ->getScalarResult();
        $totalRecords = count($hepsi);

        $qb = $em->createQueryBuilder();
        $hepsi=$qb->select('e.id')
            ->from(ESSablonlar::class, 'e')
            ->join('e.modul','m')
            ->where($sql)
            ->getQuery()
            ->getScalarResult();

        $totalRecordwithFilter = count($hepsi);
        $qb = $em->createQueryBuilder();
        $veriler=$qb->select('e.id,e.tip,e.adi,m.adi as mAdi,e.degiskenler')
            ->from(ESSablonlar::class, 'e')
            ->join('e.modul','m')
            ->where($sql)
            ->setFirstResult($row)
            ->setMaxResults($rowperpage)
            ->getQuery()
            ->getScalarResult();

        $data = array();

        foreach ($veriler as $veri){


                $degiskenler = '';
                foreach ($this->d2ArrayConvert($veri['degiskenler']) as $key=>$ddd){
                    if ($ddd!=''){
                        $degiskenler .='<span class="badge bg-label-success me-1 mb-1">'.$key.'</span>';
                    }
                }

                $islem = $this->islemler($veri['id']);

                $data[] = array(
                    "id" => $veri['id'],
                    "adi" => $veri['adi'],
                    "tip" => $veri['tip']==1?'Sms':'E-Posta',
                    "modul"=> $veri['mAdi'],
                    "degiskenler" => $degiskenler,
                    "islem" => $islem,
                );

        }

        $response = array(
            "draw" => intval($draw),
            "recordsTotal" => $totalRecords,
            "recordsFiltered" => $totalRecordwithFilter,
            "data" => $data
        );

        return $this->json($response);
    }

    #[Route('/remove', name: 'app_esSablonlar_remove')]
    public function remove(Request $request,EntityManagerInterface $em): Response
    {
        $this->dilKontrol();

        $id = $request->request->get('id');

        $veri = $em->getRepository(ESSablonlar::class)->find($id);
        $veri->setRemove(1);
        $em->persist($veri);
        $em->flush();

        return new Response('');
    }

    #[Route('/modal', name: 'app_esSablonlar_modal')]
    public function modal(Request $request,EntityManagerInterface $em): Response
    {
        $this->dilKontrol();

        $id = $request->request->get('id');

        $veriler=[];
        if($id!="add"){
            $qb = $em->createQueryBuilder();
            $veriler=$qb->select('e.id,e.adi,e.tip,e.degiskenler,m.id as mId')
                ->from(ESSablonlar::class, 'e')
                ->where('e.id='.$id)
                ->join('e.modul','m')
                ->getQuery()
                ->getScalarResult();
        }

        $qb = $em->createQueryBuilder();
        $moduller=$qb->select('i.adi as label,i.id as value')
            ->from(Izinler::class, 'i')
            ->getQuery()
            ->getScalarResult();

        return $this->render('panel/ayarlar/es_sablonlar/modal.html.twig',array('veriler'=>$veriler,'id'=>$id,'moduller'=>$moduller));
    }
    #[Route('/modalSave', name: 'app_esSablonlar_modalSave')]
    public function modalSave(Request $request,EntityManagerInterface $em,UserPasswordHasherInterface $passwordEncoder,MailerInterface $mailer, RequestStack $session): Response
    {
        $this->dilKontrol();



        $id = $request->request->get('id');

        $adi = $request->request->get('adi');
        $modul = $request->request->get('modul');
        $tip = $request->request->get('tip');
        $degiskenSay = $request->request->get('degiskenSay');

        $yDegiskenler = [];
        for($i=0;$i<$degiskenSay;$i++){
            $dd = explode(',',$request->request->get('degiskenler'.$i));
            $yDegiskenler[$dd[1]]=$dd[0];
        }



        if($id!="add"){
            $data = $em->getRepository(ESSablonlar::class)->find($id);
        } else {
            $data = new ESSablonlar();
        }

        $modul = $em->getReference(Izinler::class,$modul);

        $data->setModul($modul);
        $data->setTip($tip);
        $data->setAdi($adi);
        $data->setDegiskenler($yDegiskenler);


        $em->persist($data);
        $em->flush();



      return new Response('');
    }

    #[Route('/degiskenBul', name: 'app_esSablonlar_degiskenBul')]
    public function degiskenBul(Request $request,ManagerRegistry $doctrine): Response
    {
        $this->dilKontrol();

        $em =  $doctrine->getManager();
        $id = $request->request->get('id');


        $qb = $em->createQueryBuilder();
        $modul=$qb->select('i.tablo')
            ->from(Izinler::class, 'i')
            ->where('i.id='.$id)
            ->getQuery()
            ->getScalarResult();


        $em =  $doctrine->getManager();
        $columns = $em->getClassMetadata("App\Entity\\".str_replace('/','\\',$modul[0]['tablo']))->getColumnNames();

        $column = [];
        foreach ($columns as $cc){
            $column[] = ['value'=>$cc];
        }


        return $this->render('panel/ayarlar/es_sablonlar/degiskenBul.html.twig',array('column'=>$column));
    }

    #[Route('/degiskenEkle', name: 'app_esSablonlar_degiskenEkle')]
    public function degiskenEkle(Request $request,EntityManagerInterface $em): Response
    {
        $this->dilKontrol();

        $sutun = $request->request->get('sutun');
        $degisken = $request->request->get('degisken');

        $degisken = '#'.$this->seo_entity($degisken).'#';




        return $this->render('panel/ayarlar/es_sablonlar/degiskenEkle.html.twig',['degisken'=>$degisken,'sutun'=>$sutun]);
    }


}