<?php

namespace App\Controller;

use App\Entity\Izinler;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\Persistence\ManagerRegistry;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Mailer\MailerInterface;
use Symfony\Component\PasswordHasher\Hasher\UserPasswordHasherInterface;
use Symfony\Component\Routing\Annotation\Route;


class IzinlerController extends BaseController
{

    #[Route('/izinler', name: 'app_izinler_homepage')]
    public function index(): Response
    {
        $this->dilKontrol();

        return $this->render('panel/izinler/index.html.twig');
    }

    #[Route('/izinler/result', name: 'app_izinler_result')]
    public function result(EntityManagerInterface $em,Request $request): Response
    {
        $this->dilKontrol();




        $draw = $request->request->get('draw');
        $row= $request->request->get('start');
        $rowperpage= $request->request->get('length');
        $searchValue= $request->request->all('search')['value'];



        $sql="i.remove is null";
        if($searchValue != ''){
            $sql .=" and (i.adi like '%$searchValue%' or i.tablo like '%$searchValue%' or i.route like '%$searchValue%' or i.olusturmaTarihi like '%$searchValue%')";
        }


        $qb = $em->createQueryBuilder();
        $hepsi=$qb->select('i.id')
            ->from(Izinler::class, 'i')
            ->where("i.remove is null")
            ->getQuery()
            ->getScalarResult();
        $totalRecords = count($hepsi);

        $qb = $em->createQueryBuilder();
        $hepsi=$qb->select('i.id')
            ->from(Izinler::class, 'i')
            ->where($sql)
            ->getQuery()
            ->getScalarResult();

        $totalRecordwithFilter = count($hepsi);
        $qb = $em->createQueryBuilder();
        $veriler=$qb->select('i.id,i.olusturmaTarihi,i.adi,i.tablo,i.route')
            ->from(Izinler::class, 'i')
            ->where($sql)
            ->setFirstResult($row)
            ->setMaxResults($rowperpage)
            ->getQuery()
            ->getScalarResult();

        $data = array();

        foreach ($veriler as $veri){
                $tarih = new \DateTime($veri['olusturmaTarihi']);

                $roller = '';
                foreach ($this->rolBul($veri['id']) as $rol){
                    if ($rol!=''){
                        $roller .='<span class="badge bg-label-success me-1 mb-1">'.$rol.'</span>';
                    }
                }

                $islem = $this->islemler($veri['id']);

                $data[] = array(
                    "adi" => $veri['adi'].'<br><small class="text-muted">'.$veri['tablo'].'</small><br><small class="text-muted">'.$veri['route'].'</small>',
                    "roller" => $roller,
                    "tarih" => $tarih->format('d.m.Y H:i'),
                    "islem" => $islem,
                );

        }

        $response = array(
            "draw" => intval($draw),
            "recordsTotal" => $totalRecords,
            "recordsFiltered" => $totalRecordwithFilter,
            "data" => $data
        );

        return $this->json($response);
    }

    #[Route('/izinler/remove', name: 'app_izinler_remove')]
    public function remove(Request $request,EntityManagerInterface $em): Response
    {
        $this->dilKontrol();

        $id = $request->request->get('id');

        $veri = $em->getRepository(Izinler::class)->find($id);
        $veri->setRemove(1);
        $em->persist($veri);
        $em->flush();


        return new Response('');
    }

    #[Route('/izinler/modal', name: 'app_izinler_modal')]
    public function modal(Request $request,EntityManagerInterface $em,ManagerRegistry $doctrine): Response
    {
        $this->dilKontrol();

        $entitiesBul = $this->entityBul('../src/Entity');
        $entities = [];
        foreach ($entitiesBul as $key=>$dir){
            if(!is_array($dir)){
                $entities[] = ['value'=>$dir];
            }

        }

        $id = $request->request->get('id');

        $veriler=[];
        if($id!="add"){
            $qb = $em->createQueryBuilder();
            $veriler=$qb->select('i.adi,i.tablo,i.route')
                ->from(Izinler::class, 'i')
                ->where('i.id='.$id)
                ->getQuery()
                ->getScalarResult();
        }



        return $this->render('panel/izinler/modal.html.twig',['veriler'=>$veriler,'id'=>$id,'entities'=>$entities]);
    }
    #[Route('/izinler/modalSave', name: 'app_izinler_modalSave')]
    public function modalSave(Request $request,EntityManagerInterface $em,UserPasswordHasherInterface $passwordEncoder,MailerInterface $mailer): Response
    {
        $this->dilKontrol();

        $id = $request->request->get('id');

        $adi = $request->request->get('adi');
        $route = $request->request->get('route');
        $tablo = $request->request->get('tablo');


        if($id!="add"){
            $data = $em->getRepository(Izinler::class)->find($id);
        }else{
            $data = new Izinler();
            $data->setOlusturmaTarihi(new \DateTime('now'));
        }

        $data->setAdi($adi);
        $data->setRoute($route);
        $data->setTablo($tablo);

        $em->persist($data);
        $em->flush();



      return new Response('');
    }



}