<?php

namespace App\Controller;

use App\Entity\Rol;
use App\Entity\User;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Mailer\MailerInterface;
use Symfony\Component\PasswordHasher\Hasher\UserPasswordHasherInterface;
use Symfony\Component\Routing\Annotation\Route;


class KullanicilarController extends BaseController
{

    #[Route('/kullanicilar', name: 'app_kullanicilar_homepage')]
    public function index(): Response
    {
        $this->dilKontrol();

        return $this->render('panel/kullanicilar/index.html.twig');
    }

    #[Route('/kullanicilar/result', name: 'app_kullanicilar_result')]
    public function result(EntityManagerInterface $em,Request $request,RequestStack $session): Response
    {
        $this->dilKontrol();




        $draw = $request->request->get('draw');
        $row= $request->request->get('start');
        $rowperpage= $request->request->get('length');
        $searchValue= $request->request->all('search')['value'];




        $sql="u.id not in(".$this->getUser()->getId().") and u.remove is null and u.roles not like '%ROLE_DEV%' and u.roles not like '%ROLE_MUSTERI%'";
        if($searchValue != ''){
            $sql .=" and (u.adsoyad like '%$searchValue%' or u.email like '%$searchValue%' or u.telefon like '%$searchValue%' or r.adi like '%$searchValue%')";
        }


        $qb = $em->createQueryBuilder();
        $hepsi=$qb->select('u.id')
            ->from(User::class, 'u')
            ->leftJoin('u.rol', 'r')
            ->where("u.id not in(".$this->getUser()->getId().") and u.remove is null and u.roles not like '%ROLE_DEV%' and u.roles not like '%ROLE_MUSTERI%'")
            ->getQuery()
            ->getScalarResult();
        $totalRecords = count($hepsi);

        $qb = $em->createQueryBuilder();
        $hepsi=$qb->select('u.id')
            ->from(User::class, 'u')
            ->leftJoin('u.rol', 'r')
            ->where($sql)
            ->getQuery()
            ->getScalarResult();

        $totalRecordwithFilter = count($hepsi);
        $qb = $em->createQueryBuilder();
        $veriler=$qb->select('u.id,u.isVerified,u.adsoyad,u.unvan,u.telefon,u.email,u.resim,r.adi as rAdi')
            ->from(User::class, 'u')
            ->leftJoin('u.rol', 'r')
            ->where($sql)
            ->setFirstResult($row)
            ->setMaxResults($rowperpage)
            ->getQuery()
            ->getScalarResult();

        $data = array();

        foreach ($veriler as $veri){

                $resim = "";
                if($veri['resim']==''){
                    $resim = '<div class="avatar avatar-md me-3"><span class="avatar-initial rounded-circle bg-label-primary">'.mb_substr(explode(' ',$veri['adsoyad'])[0],0,1,'UTF-8').mb_substr(explode(' ',$veri['adsoyad'])[1]??"",0,1,'UTF-8').'</span></div>';
                }else{
                    $resim = '<div class="avatar avatar-md me-3">'.$this->imgOlustur($veri['resim'],48,100).'</div>';
                }


                $islem = $this->islemler($veri['id']);

                $data[] = array(
                    "kullanici" => '<div class="d-flex">'.$resim.'<div>'.$veri['adsoyad'].'<br><span class="badge bg-label-primary">'.$veri['unvan'].'</span></div>',
                    "eposta" => $veri['email'],
                    "telefon" => $veri['telefon'],
                    "rol"=>'<span class="badge bg-label-info">'.$veri['rAdi'].'</span>',
                    "durum" => $veri['isVerified'] ? '<span class="badge bg-label-success">Aktif</span>' : '<span class="badge bg-label-danger">Pasif</span>',

                    "islem" => $islem,
                );

        }

        $response = array(
            "draw" => intval($draw),
            "recordsTotal" => $totalRecords,
            "recordsFiltered" => $totalRecordwithFilter,
            "data" => $data
        );

        return $this->json($response);
    }

    #[Route('/kullanicilar/remove', name: 'app_kullanicilar_remove')]
    public function remove(Request $request,EntityManagerInterface $em): Response
    {
        $this->dilKontrol();

        $id = $request->request->get('id');

        $veri = $em->getRepository(User::class)->find($id);
        $veri->setRemove(1);
        $veri->setIsVerified(0);
        $veri->setEmail('silinmis'.date('U').'@silinmis.com');
        $em->persist($veri);
        $em->flush();

        return new Response('');
    }

    #[Route('/kullanicilar/modal', name: 'app_kullanicilar_modal')]
    public function modal(Request $request,EntityManagerInterface $em,RequestStack $session): Response
    {
        $this->dilKontrol();

        $id = $request->request->get('id');

        $veriler=[];
        if($id!="add"){
            $qb = $em->createQueryBuilder();
            $veriler=$qb->select('r,rol.id as rId')
                ->from(User::class, 'r')
                ->leftJoin('r.rol','rol')
                ->where('r.id='.$id)
                ->getQuery()
                ->getScalarResult();
        }


        $qb = $em->createQueryBuilder();
        $roller=$qb->select('r.adi as label,r.id as value,r.code')
            ->from('App\Entity\Rol', 'r')
            ->where('r.remove is null')
            ->getQuery()
            ->getScalarResult();

        return $this->render('panel/kullanicilar/modal.html.twig',array('veriler'=>$veriler,'id'=>$id,'roller'=>$roller));
    }
    #[Route('/kullanicilar/modalSave', name: 'app_kullanicilar_modalSave')]
    public function modalSave(Request $request,EntityManagerInterface $em,UserPasswordHasherInterface $passwordEncoder,MailerInterface $mailer, RequestStack $session): Response
    {
        $this->dilKontrol();

        $id = $request->request->get('id');

        $adsoyad = $request->request->get('adsoyad');
        $unvan = $request->request->get('unvan');
        $eposta = $request->request->get('eposta');
        $telefon = $request->request->get('telefon');
        $rol = $request->request->get('rol');
        $durum = $request->request->get('durum');
        $sifre = $request->request->get('sifre');

        $rol = $em->getRepository(Rol::class)->find($rol);

        $sql = "";
        if($id!='add'){
            $sql = " and k.id not in($id)";
        }

        $qb=$em->createQueryBuilder();
        $kullaniciTara=$qb->select('k.id')
            ->from(User::class, 'k')
            ->where("k.email='".$eposta."'".$sql)
            ->getQuery()
            ->getScalarResult();


        if(count($kullaniciTara)>0){
            return new Response('E-Posta adresi başkası tarafından kullanılıyor.');
        }else{
            if($id=='add'){
                $user = new User();
                $sifre = $this->rasgele(8);
                $user->setPassword($passwordEncoder->hashPassword($user, $sifre));
            }else{
                $user = $em->getRepository(User::class)->find($id);
                $sifre = $request->request->get('sifre');
                if($sifre!=''){
                    $user->setPassword($passwordEncoder->hashPassword($user, $sifre));
                }else{
                    $sifre = "Aynı";
                }
            }

            $user->setUnvan($unvan);
            $user->setAdsoyad($adsoyad);
            $user->setEmail($eposta);
            $user->setTelefon($telefon);
            $user->setRol($rol);
            $user->setRoles(array($rol->getCode()));
            $user->setIsVerified($durum);
            if($id=='add') {
                $user->setIsPassword(0);
            }
            $em->persist($user);
            $em->flush();



            $data = "";
            $sablonBul = $this->sablonBul(($id=='add'?'1':'2'),$user->getId());


            if($sablonBul!='') {
                $data = $sablonBul['sablon']['icerik'];
                $search = [];
                $replace = [];
                foreach ($this->d2ArrayConvert($sablonBul['sablon']['degiskenler']) as $key => $degisken) {
                    if ($key == '#sifre#') {
                        $dd = $sifre;
                    } else {
                        $dd = $sablonBul['dbTablo'][$degisken];
                    }
                    $search[] = $key;
                    $replace[] = $dd;
                }


                $data = str_replace($search, $replace, $data);

            

                $this->mailGonder($eposta,$sablonBul['sablon']['konu'],$this->renderView('mail/sablon.html.twig', array('data' => $data, 'konu' => $sablonBul['sablon']['konu'])));
            }
        }
      return new Response('');
    }



}