<?php

namespace App\Controller;
use App\Entity\Dil;
use App\Entity\Odemeler;
use App\Entity\ParaBirimleri;
use App\Entity\Raporlar;
use App\Entity\Siteler;
use App\Entity\User;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\Persistence\ManagerRegistry;
use Google\AdsApi\AdManager\v202311\Site;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;

class MainController extends BaseController
{

    #[Route('/', name: 'app_homepage')]
    public function index(Request $request,EntityManagerInterface $em,RequestStack $session): Response
    {
        $this->dilKontrol();

        $sql = '';
        if($this->getUser()->getRoles()[0]=='ROLE_MUSTERI'){
            $sql.=' and s.musteri='.$this->getUser()->getId();
        }

        $qb = $em->createQueryBuilder();
        $siteler = $qb->select('s.id as value, s.url as label')
            ->from(Siteler::class, 's')
            ->where("s.remove is null".$sql)
            ->getQuery()
            ->getScalarResult();

        return $this->render('panel/anasayfa/anasayfa.html.twig',['siteler'=>$siteler]);

    }
    #[Route('/dil-degistir', name: 'app_dilDegistir')]
    public function dilDegistir(Request $request,EntityManagerInterface $em,RequestStack $session): Response
    {
        $this->dilKontrol();

        $kisa = $request->get("kisa");


        $qb = $em->createQueryBuilder();
        $diller=$qb->select('d.id,d.kisa,d.uzun')
            ->from(Dil::class, 'd')
            ->where("d.kisa='$kisa'")
            ->getQuery()
            ->getScalarResult();

        $session->getSession()->set('dil',$diller[0]);


        return new Response('');
    }
    #[Route('/istatistikler', name: 'app_istatistikler')]
    public function istatistikler(Request $request,EntityManagerInterface $em,RequestStack $session): Response
    {
        $this->dilKontrol();


        $site = $request->get('site');
        $sql = '';
        if($site!=''){
            $sql.=' and r.site='.$site;
        }

        $suan = new \DateTime('now');
        // bugün

        $qb = $em->createQueryBuilder();
        $raporlarBugun = $qb->select('sum(r.adxReservationPubCostDelivered) as toplam')
            ->from(Raporlar::class, 'r')
            ->where("r.thisDate='".$suan->format('Y-m-d')."'".$sql)
            ->getQuery()
            ->getScalarResult();

        // Dün
        $qb = $em->createQueryBuilder();
        $raporlarDun = $qb->select('sum(r.adxReservationPubCostDelivered) as toplam')
            ->from(Raporlar::class, 'r')
            ->where("r.thisDate='".(new \DateTime('now'))->modify('-1 days')->format('Y-m-d')."'".$sql)
            ->getQuery()
            ->getScalarResult();

        // Son 7 gün
        $son7 = (new \DateTime('now'))->modify('-7 days');
        $qb = $em->createQueryBuilder();
        $raporlarSon7 = $qb->select('sum(r.adxReservationPubCostDelivered) as toplam')
            ->from(Raporlar::class, 'r')
            ->where("r.thisDate >= '".$son7->format('Y-m-d')."' and r.thisDate <='".$suan->format('Y-m-d')."'".$sql)
            ->getQuery()
            ->getScalarResult();


        // Son 30 gün
        $son30 = (new \DateTime('now'))->modify('-30 days');
        $qb = $em->createQueryBuilder();
        $raporlarSon30 = $qb->select('sum(r.adxReservationPubCostDelivered) as toplam, avg(r.adxReservationEcpmDelivered) as cpmOrt, avg(r.activeViewAdxViewableImpressionsRate) as rateOrt')
            ->from(Raporlar::class, 'r')
            ->where("r.thisDate >= '".$son30->format('Y-m-d')."' and r.thisDate <='".$suan->format('Y-m-d')."'".$sql)
            ->getQuery()
            ->getScalarResult();



        //son 1 yıl
        if($site!=''){
            $site = 'AND r.site_id='.$site;
        }

        $son1Yil = (new \DateTime('now'))->modify('-1 years');
        $sql = "
                SELECT 
                    DATE_FORMAT(r.this_date, '%m/%Y') as ay, 
                    SUM(r.adx_reservation_pub_cost_delivered) as toplamKazanc, 
                    SUM(r.adx_reservation_impressions_delivered) as toplamGoruntulenme
                FROM raporlar r
                WHERE r.this_date >= :son1Yil 
                AND r.this_date <= :suan 
                $site
                GROUP BY ay
            ";

        $conn = $em->getConnection();
        $stmt = $conn->prepare($sql);


        $raporlarSon1Yil = $stmt->execute([
            'son1Yil' => $son1Yil->format('Y-m-d'),
            'suan' => $suan->format('Y-m-d')
        ])->fetchAllAssociative();

        $ttBol = explode('/',$raporlarSon1Yil[count($raporlarSon1Yil) - 1]['ay']);
        $sonTarih = new \DateTime($ttBol[1].'-'.$ttBol[0].'-01');

        for ($i = count($raporlarSon1Yil); $i < 12; $i++) {
            // Bir ay geriye git
            $sonTarih->modify('-1 month');

            // Geriye gidilen tarihi 'm/Y' formatında ekle
            $raporlarSon1Yil[] = [
                'ay' => $sonTarih->format('m/Y'),
                'toplamKazanc' => 0,
                'toplamGoruntulenme' => 0
            ];
        }


        return $this->render('panel/anasayfa/istatistikler.html.twig',['bugun'=>$raporlarBugun,'raporlarDun'=>$raporlarDun,'son7'=>$raporlarSon7,'son30'=>$raporlarSon30,'son1Yil'=>$raporlarSon1Yil]);
    }

    #[Route('/bildirimGonder', name: 'app_bildirimGonder')]
    public function bildirimGonder(Request $request,EntityManagerInterface $em,RequestStack $session): Response
    {
        $this->dilKontrol();

        $qb = $em->createQueryBuilder();
        $mobilBildirimler = $qb->select('b.id,b.baslik,b.metin,b.tarih,b.musteriler')
            ->from(MobilBildirimler::class, 'b')
            ->where('b.durum=0')
            ->getQuery()
            ->getScalarResult();



        foreach ($mobilBildirimler as $bildirim){
            $ok = 1;

            if($bildirim['tarih']!=null){
                $suan = new \DateTime('now');
                $tarih = new \DateTime($bildirim['tarih']);
                if($suan->format('U')<$tarih->format('U')){
                    $ok = 0;
                }
            }

            if($ok==1) {
                $tokens = [];
                $kullanicilar = $this->d2ArrayConvert($bildirim['musteriler']);
                if ($kullanicilar == "") {


                    $qb = $em->createQueryBuilder();
                    $tokenssss = $qb->select('t.token,k.id')
                        ->from(Tokens::class, 't')
                        ->leftjoin('t.kullanici', 'k')
                        ->getQuery()
                        ->getScalarResult();
                    foreach ($tokenssss as $tt) {
                        $tokens[] = ['token'=>$tt['token'],'id'=>$tt['id']];
                    }

                }else{
                    foreach ($kullanicilar as $k){
                        $qb = $em->createQueryBuilder();
                        $tokenssss = $qb->select('t.token')
                            ->from(Tokens::class, 't')
                            ->leftjoin('t.kullanici', 'k')
                            ->where('k.id='.$k)
                            ->getQuery()
                            ->getScalarResult();
                        foreach ($tokenssss as $tt) {
                            $tokens[] = ['token'=>$tt['token'],'id'=>$k];
                        }
                    }
                }



                foreach ($tokens as $tt) {
                    if($tt['id']!=''){

                        $qb = $em->createQueryBuilder();
                        $bildirimSor = $qb->select('b.id')
                            ->from(Bildirimler::class, 'b')
                            ->where('b.mobilBildirim='.$bildirim['id'].' and b.kullanici='.$tt['id'])
                            ->setMaxResults(1)
                            ->getQuery()
                            ->getScalarResult();

                        if(count($bildirimSor)==0) {
                            $uu = $em->getReference(User::class, $tt['id']);
                            $mb = $em->getReference(MobilBildirimler::class, $bildirim['id']);

                            $bb = new Bildirimler();
                            $bb->setBaslik($bildirim['baslik']);
                            $bb->setMetin($bildirim['metin']);
                            $bb->setTarih(new \DateTime('now'));
                            $bb->setOkuma(0);
                            $bb->setKullanici($uu);
                            $bb->setMobilBildirim($mb);
                            $bb->setYonlendirme("");
                            $em->persist($bb);
                            $em->flush();



                        }
                    }

                }


                $this->mobilBildirimGonder($bildirim['baslik'], $bildirim['metin'], $tokens,);

                $mbb = $em->getRepository(MobilBildirimler::class)->find($bildirim['id']);
                $mbb->setDurum(1);
                $em->persist($mbb);
                $em->flush();
            }
        }

        return new Response('');
    }


    #[Route('/otoKurGuncelle', name: 'app_otoKurGuncelle')]
    public function otoKurGuncelle(ManagerRegistry $doctrine): Response
    {
        $this->dilKontrol();
        // Burası cron ile çalıştırılacak!

        $em =  $doctrine->getManager();
    

        $veriler = $em->getRepository(ParaBirimleri::class)->findBy(['guncelleme'=>1]);
        foreach ($veriler as $veri){
            $veri->setKur($this->kurGuncelle($veri->getApiUrl(),$veri->getKurTipi(),'TRY',$veri->getKisaAdi()));
            $em->persist($veri);
            $em->flush();
        } 
  

        return new Response('');
    }


    #[Route('/ilGetir', name: 'app_ilGetir')]
    public function ilGetir(Request $request, ManagerRegistry $doctrine): Response
    {
        $this->dilKontrol();

        $em = $doctrine->getManager();

        $id = $request->request->get('id');


        $qb = $em->createQueryBuilder();
        $veriler=$qb->select('s.id as value,s.adi as label')
            ->from(Il::class, 's')
            ->join('s.ulke','ulke')
            ->where('ulke.id='.$id)
            ->getQuery()
            ->getScalarResult();



        $data = '<option value="">İl Seçiniz</option>';
        foreach ($veriler as $veri){
            $data .= '<option value="'.$veri['value'].'">'.$veri['label'].'</option>';
        }


        return new Response($data);

    }

    #[Route('/ilceGetir', name: 'app_ilceGetir')]
    public function ilceGetir(Request $request, ManagerRegistry $doctrine): Response
    {
        $this->dilKontrol();

        $em = $doctrine->getManager();

        $id = $request->request->get('id');


        $qb = $em->createQueryBuilder();
        $veriler=$qb->select('s.id as value,s.adi as label')
            ->from(Ilce::class, 's')
            ->join('s.il','il')
            ->where('il.id='.$id)
            ->getQuery()
            ->getScalarResult();



        $data = '<option value="">İlçe Seçiniz</option>';
        foreach ($veriler as $veri){
            $data .= '<option value="'.$veri['value'].'">'.$veri['label'].'</option>';
        }


        return new Response($data);

    }
    #[Route('/semtGetir', name: 'app_semtGetir')]
    public function semtGetir(Request $request, ManagerRegistry $doctrine): Response
    {
        $this->dilKontrol();

        $em = $doctrine->getManager();
        $id = $request->request->get('id');

        $qb = $em->createQueryBuilder();
        $veriler=$qb->select('s.id as value,s.adi as label')
            ->from(Semt::class, 's')
            ->join('s.ilce','ilce')
            ->where('ilce.id='.$id)
            ->getQuery()
            ->getScalarResult();



        $data = '<option value="">Semt Seçiniz</option>';
        foreach ($veriler as $veri){
            $data .= '<option value="'.$veri['value'].'">'.$veri['label'].'</option>';
        }


        return new Response($data);

    }
    #[Route('/mahalleGetir', name: 'app_mahalleGetir')]
    public function mahalleGetir(Request $request, ManagerRegistry $doctrine): Response
    {
        $this->dilKontrol();
        $em = $doctrine->getManager();
        $id = $request->request->get('id');

        $qb = $em->createQueryBuilder();
        $veriler=$qb->select('s.id as value,s.adi as label, s.postaKodu')
            ->from(Mahalle::class, 's')
            ->join('s.semt','semt')
            ->where('semt.id='.$id)
            ->getQuery()
            ->getScalarResult();


        $data = '<option value="">Mahalle Seçiniz</option>';
        foreach ($veriler as $veri){
            $data .= '<option value="'.$veri['value'].'">'.$veri['label'].' - '.$veri['postaKodu'].'</option>';
        }


        return new Response($data);

    }

    #[Route('/sokakGetir', name: 'app_sokakGetir')]
    public function sokakGetir(Request $request, ManagerRegistry $doctrine): Response
    {
        $this->dilKontrol();
        $em = $doctrine->getManager();
        $id = $request->request->get('id');

        $qb = $em->createQueryBuilder();
        $veriler=$qb->select('s.id as value,s.adi as label')
            ->from(Sokak::class, 's')
            ->join('s.mahalle','mahalle')
            ->where('mahalle.id='.$id)
            ->getQuery()
            ->getScalarResult();


        $data = '<option value="">Sokak Seçiniz</option>';
        foreach ($veriler as $veri){
            $data .= '<option value="'.$veri['value'].'">'.$veri['label'].'</option>';
        }


        return new Response($data);

    }


    #[Route('/search', name: 'app_search')]
    public function search(Request $request, ManagerRegistry $doctrine,RequestStack $session): Response
    {
        $this->dilKontrol();
        $data = [];
        $sistemIzinler = $session->getSession()->get('sistem_izinler');
        $userRoles = $this->getUser()->getRoles()[0];



        //Sayfalar
        $i=0;
        $data['veriler'][$i]['name'] = "Profil";
        $data['veriler'][$i]['url'] = "/profil";
        $data['veriler'][$i]['icon'] = 'ti-user';
        $i++;

        
     

        if($this->yetkiBak('/kullanicilar',$userRoles,$sistemIzinler)==1) {
            $data['veriler'][$i]['name'] = "Kullanıcılar";
            $data['veriler'][$i]['url'] = "/kullanicilar";
            $data['veriler'][$i]['icon'] = 'ti-users';
            $i++;
        }
        if($this->yetkiBak('/roller',$userRoles,$sistemIzinler)==1) {
            $data['veriler'][$i]['name'] = "Roller";
            $data['veriler'][$i]['url'] = "/roller";
            $data['veriler'][$i]['icon'] = 'ti-sign-right';
            $i++;
        }
        if($this->yetkiBak('/izinler',$userRoles,$sistemIzinler)==1) {
            $data['veriler'][$i]['name'] = "Modüller";
            $data['veriler'][$i]['url'] = "/izinler";
            $data['veriler'][$i]['icon'] = 'ti-prison';
            $i++;
        }


      
        // Ayarlar
        if($this->yetkiBak('/ayarlar/genel-ayarlar',$userRoles,$sistemIzinler)==1) {
            $data['veriler'][$i]['name'] = "Genel Ayarlar";
            $data['veriler'][$i]['url'] = "/ayarlar/genel-ayarlar";
            $data['veriler'][$i]['icon'] = 'ti-settings';
            $i++;
        }
        if($this->yetkiBak('/ayarlar/smtp-ayarlari',$userRoles,$sistemIzinler)==1) {
            $data['veriler'][$i]['name'] = "SMTP Ayarları";
            $data['veriler'][$i]['url'] = "/ayarlar/smtp-ayarlari";
            $data['veriler'][$i]['icon'] = 'ti-settings';
            $i++;
        }
        if($this->yetkiBak('/ayarlar/eposta-sablonlari',$userRoles,$sistemIzinler)==1) {
            $data['veriler'][$i]['name'] = "E-Posta Şablonları";
            $data['veriler'][$i]['url'] = "/ayarlar/eposta-sablonlari";
            $data['veriler'][$i]['icon'] = 'ti-settings';
            $i++;
        }


        if($this->yetkiBak('/ayarlar/es-sablonlar',$userRoles,$sistemIzinler)==1) {
            $data['veriler'][$i]['name'] = "Şablon Değişkenleri";
            $data['veriler'][$i]['url'] = "/ayarlar/es-sablonlar";
            $data['veriler'][$i]['icon'] = 'ti-brush';
            $i++;
        }




        if($this->yetkiBak('/ayarlar/global-embed',$userRoles,$sistemIzinler)==1) {
            $data['veriler'][$i]['name'] = "Global Embed";
            $data['veriler'][$i]['url'] = "/ayarlar/global-embed";
            $data['veriler'][$i]['icon'] = 'ti-squares-diagonal';
            $i++;
        }


        if($this->yetkiBak('/odemeler',$userRoles,$sistemIzinler)==1) {
            $data['veriler'][$i]['name'] = "Ödemeler";
            $data['veriler'][$i]['url'] = "/odemeler";
            $data['veriler'][$i]['icon'] = 'ti-wallet';
            $i++;
        }

        if($this->yetkiBak('/siteler',$userRoles,$sistemIzinler)==1) {
            $data['veriler'][$i]['name'] = "Siteler";
            $data['veriler'][$i]['url'] = "/siteler";
            $data['veriler'][$i]['icon'] = 'ti-world';
            $i++;
        }

        if($this->yetkiBak('/musteriler',$userRoles,$sistemIzinler)==1) {
            $data['veriler'][$i]['name'] = "Müşteriler";
            $data['veriler'][$i]['url'] = "/musteriler";
            $data['veriler'][$i]['icon'] = 'ti-briefcase';
            $i++;
        }



        if($this->yetkiBak('/tanimlamalar/bankalar',$userRoles,$sistemIzinler)==1) {
            $data['veriler'][$i]['name'] = "Bankalar";
            $data['veriler'][$i]['url'] = "/tanimlamalar/bankalar";
            $data['veriler'][$i]['icon'] = 'ti-adjustments-alt';
               $i++;
        }

        if($this->yetkiBak('/tanimlamalar/para-birimleri',$userRoles,$sistemIzinler)==1) {
            $data['veriler'][$i]['name'] = "Para Birimleri";
            $data['veriler'][$i]['url'] = "/tanimlamalar/para-birimleri";
            $data['veriler'][$i]['icon'] = 'ti-coins';
               $i++;
        }



        return $this->json($data);

    }


    #[Route('/otomatik-odeme', name: 'app_otomatik_odeme')]
    public function otomatikOdeme(Request $request,EntityManagerInterface $em,RequestStack $session): Response
    {
        $this->dilKontrol();


        $suan = new \DateTime('now');

        $qb = $em->createQueryBuilder();
        $siteler = $qb->select('s')
            ->from(Siteler::class, 's')
            ->where("s.remove is null")
            ->getQuery()
            ->getScalarResult();


        foreach ($siteler as $site){
            $qb = $em->createQueryBuilder();
            $odemeler = $qb->select('o')
                ->from(Odemeler::class, 'o')
                ->orderBy('o.id','DESC')
                ->setMaxResults(1)
                ->getQuery()
                ->getScalarResult();

            $fark = date_diff($suan,$odemeler[0]['o_tarih']);

            if($fark->days >= $site['s_vade']){

                $qb = $em->createQueryBuilder();
                $raporlar = $qb->select('SUM(r.adxReservationPubCostDelivered) as toplam')
                    ->from(Raporlar::class, 'r')
                    ->join('r.site','s')
                    ->where("r.thisDate > '".$odemeler[0]['o_tarih']->format('Y-m-d')."'")
                    ->getQuery()
                    ->getScalarResult();





                $data = new Odemeler();
                $data->setTarih(new \DateTime('now'));

                $data->setDonem($suan->format('m').'/'.$suan->format('Y'));
                $data->setSite($em->getReference(Siteler::class, $site['s_id']));
                $data->setKazanc($raporlar[0]['toplam']);
                $data->setDurum(0);

                $em->persist($data);
                $em->flush();




            }


        }


        return new Response('');
    }


}