<?php

namespace App\Controller;

use App\Entity\BankaBilgileri;
use App\Entity\Bankalar;
use App\Entity\FaturaBilgileri;
use App\Entity\Rol;
use App\Entity\User;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Mailer\MailerInterface;
use Symfony\Component\PasswordHasher\Hasher\UserPasswordHasherInterface;
use Symfony\Component\Routing\Annotation\Route;


class MusterilerController extends BaseController
{

    #[Route('/musteriler', name: 'app_musteriler_homepage')]
    public function index(): Response
    {
        $this->dilKontrol();

        return $this->render('panel/musteriler/index.html.twig');
    }

    #[Route('/musteriler/result', name: 'app_musteriler_result')]
    public function result(EntityManagerInterface $em,Request $request,RequestStack $session): Response
    {
        $this->dilKontrol();




        $draw = $request->request->get('draw');
        $row= $request->request->get('start');
        $rowperpage= $request->request->get('length');
        $searchValue= $request->request->all('search')['value'];




        $sql="u.remove is null and u.roles like '%ROLE_MUSTERI%'";
        if($searchValue != ''){
            $sql .=" and (u.adsoyad like '%$searchValue%' or u.email like '%$searchValue%' or u.telefon like '%$searchValue%' or r.adi like '%$searchValue%')";
        }


        $qb = $em->createQueryBuilder();
        $hepsi=$qb->select('u.id')
            ->from(User::class, 'u')
            ->leftJoin('u.rol', 'r')
            ->where("u.remove is null and u.roles like '%ROLE_MUSTERI%'")
            ->getQuery()
            ->getScalarResult();
        $totalRecords = count($hepsi);

        $qb = $em->createQueryBuilder();
        $hepsi=$qb->select('u.id')
            ->from(User::class, 'u')
            ->leftJoin('u.rol', 'r')
            ->where($sql)
            ->getQuery()
            ->getScalarResult();

        $totalRecordwithFilter = count($hepsi);
        $qb = $em->createQueryBuilder();
        $veriler=$qb->select('u.id,u.isVerified,u.adsoyad,u.unvan,u.telefon,u.email,u.resim,r.adi as rAdi')
            ->from(User::class, 'u')
            ->leftJoin('u.rol', 'r')
            ->where($sql)
            ->setFirstResult($row)
            ->setMaxResults($rowperpage)
            ->getQuery()
            ->getScalarResult();

        $data = array();

        foreach ($veriler as $veri){

                $resim = "";
                if($veri['resim']==''){
                    $resim = '<div class="avatar avatar-md me-3"><span class="avatar-initial rounded-circle bg-label-primary">'.mb_substr(explode(' ',$veri['adsoyad'])[0],0,1,'UTF-8').mb_substr(explode(' ',$veri['adsoyad'])[1]??"",0,1,'UTF-8').'</span></div>';
                }else{
                    $resim = '<div class="avatar avatar-md me-3">'.$this->imgOlustur($veri['resim'],48,100).'</div>';
                }

                $islem = '<div class="d-flex"> <a href="javascript:void(0);" onclick="bankaBilgileriAc('.$veri['id'].');" class="btn btn-warning btn-sm me-2">'.$this->ceviri('Banka Bilgileri').'</a> <a href="javascript:void(0);" class="btn btn-primary btn-sm me-2" onclick="faturaBilgileriAc('.$veri['id'].');">'.$this->ceviri('Fatura Bilgileri').'</a> '.$this->islemler($veri['id']).'</div>';


                $data[] = array(
                    "kullanici" => '<div class="d-flex">'.$resim.'<div>'.$veri['adsoyad'].'<br><span class="badge bg-label-primary">'.$veri['unvan'].'</span></div>',
                    "eposta" => $veri['email'],
                    "telefon" => $veri['telefon'],
                    "durum" => $veri['isVerified'] ? '<span class="badge bg-label-success">'.$this->ceviri('Aktif').'</span>' : '<span class="badge bg-label-danger">'.$this->ceviri('Pasif').'</span>',
                    "islem" => $islem,
                );

        }

        $response = array(
            "draw" => intval($draw),
            "recordsTotal" => $totalRecords,
            "recordsFiltered" => $totalRecordwithFilter,
            "data" => $data
        );

        return $this->json($response);
    }

    #[Route('/musteriler/remove', name: 'app_musteriler_remove')]
    public function remove(Request $request,EntityManagerInterface $em): Response
    {
        $this->dilKontrol();

        $id = $request->request->get('id');

        $veri = $em->getRepository(User::class)->find($id);
        $veri->setRemove(1);
        $veri->setIsVerified(0);
        $veri->setEmail('silinmis'.date('U').'@silinmis.com');
        $em->persist($veri);
        $em->flush();

        return new Response('');
    }

    #[Route('/musteriler/modal', name: 'app_musteriler_modal')]
    public function modal(Request $request,EntityManagerInterface $em,RequestStack $session): Response
    {
        $this->dilKontrol();

        $id = $request->request->get('id');

        $veriler=[];
        if($id!="add"){
            $qb = $em->createQueryBuilder();
            $veriler=$qb->select('r')
                ->from(User::class, 'r')
                ->where('r.id='.$id)
                ->getQuery()
                ->getScalarResult();
        }




        return $this->render('panel/musteriler/modal.html.twig',array('veriler'=>$veriler,'id'=>$id));
    }
    #[Route('/musteriler/modalSave', name: 'app_musteriler_modalSave')]
    public function modalSave(Request $request,EntityManagerInterface $em,UserPasswordHasherInterface $passwordEncoder,MailerInterface $mailer, RequestStack $session): Response
    {
        $this->dilKontrol();

        $id = $request->request->get('id');

        $adsoyad = $request->request->get('adsoyad');
        $unvan = $request->request->get('unvan');
        $eposta = $request->request->get('eposta');
        $telefon = $request->request->get('telefon');
        $durum = $request->request->get('durum');
        $sifre = $request->request->get('sifre');
        $ekstraEmbed = $request->request->get('ekstraEmbed');


        $sql = "";
        if($id!='add'){
            $sql = " and k.id not in($id)";
        }

        $qb=$em->createQueryBuilder();
        $kullaniciTara=$qb->select('k.id')
            ->from(User::class, 'k')
            ->where("k.email='".$eposta."'".$sql)
            ->getQuery()
            ->getScalarResult();


        if(count($kullaniciTara)>0){
            return new Response('E-Posta adresi başkası tarafından kullanılıyor.');
        }else{
            if($id=='add'){
                $user = new User();
                $sifre = $this->rasgele(8);
                $user->setPassword($passwordEncoder->hashPassword($user, $sifre));
            }else{
                $user = $em->getRepository(User::class)->find($id);
                $sifre = $request->request->get('sifre');
                if($sifre!=''){
                    $user->setPassword($passwordEncoder->hashPassword($user, $sifre));
                }else{
                    $sifre = "Aynı";
                }
            }

            $user->setEkstraEmbed($ekstraEmbed==''?null : $ekstraEmbed);
            $user->setUnvan($unvan);
            $user->setAdsoyad($adsoyad);
            $user->setEmail($eposta);
            $user->setTelefon($telefon);
            $user->setRol(null);
            $user->setRoles(['ROLE_MUSTERI']);
            $user->setIsVerified($durum);
            if($id=='add') {
                $user->setIsPassword(0);
            }
            $em->persist($user);
            $em->flush();



            $data = "";
            $sablonBul = $this->sablonBul(($id=='add'?'1':'2'),$user->getId());


            if($sablonBul!='') {
                $data = $sablonBul['sablon']['icerik'];
                $search = [];
                $replace = [];
                foreach ($this->d2ArrayConvert($sablonBul['sablon']['degiskenler']) as $key => $degisken) {
                    if ($key == '#sifre#') {
                        $dd = $sifre;
                    } else {
                        $dd = $sablonBul['dbTablo'][$degisken];
                    }
                    $search[] = $key;
                    $replace[] = $dd;
                }


                $data = str_replace($search, $replace, $data);



                $this->mailGonder($eposta,$sablonBul['sablon']['konu'],$this->renderView('mail/sablon.html.twig', array('data' => $data, 'konu' => $sablonBul['sablon']['konu'])));
            }
        }
      return new Response('');
    }


    #[Route('/musteriler/bankaBilgileriAc', name: 'app_musteriler_bankaBilgileriAc')]
    public function bankaBilgileriAc(Request $request,EntityManagerInterface $em,RequestStack $session): Response
    {
        $this->dilKontrol();

        $id = $request->request->get('id');


        $qb = $em->createQueryBuilder();
        $veriler=$qb->select('b,ban.id as bId')
            ->from(BankaBilgileri::class, 'b')
            ->join('b.musteri','m')
            ->join('b.banka','ban')
            ->where('m.id='.$id)
            ->getQuery()
            ->getScalarResult();


        $qb = $em->createQueryBuilder();
        $bankalar=$qb->select('b.id as value, b.adi as label')
            ->from(Bankalar::class, 'b')
            ->where('b.remove is null')
            ->getQuery()
            ->getScalarResult();





        return $this->render('panel/musteriler/bankaBilgileri.html.twig',array('veriler'=>$veriler,'bankalar'=>$bankalar,'id'=>$id));
    }

    #[Route('/musteriler/bankaBilgileriKaydet', name: 'app_musteriler_bankaBilgileriKaydet')]
    public function bankaBilgileriKaydet(Request $request,EntityManagerInterface $em,UserPasswordHasherInterface $passwordEncoder,MailerInterface $mailer, RequestStack $session): Response
    {
        $this->dilKontrol();

        $id = $request->request->get('id');

        $banka = $request->request->get('banka'); $banka = $em->getReference(Bankalar::class,$banka);
        $hesapNo = $request->request->get('hesapNo');
        $hesapSahibi = $request->request->get('hesapSahibi');
        $iban = $request->request->get('iban');



        $bankaBilgileri = $em->getRepository(BankaBilgileri::class)->findBy(['musteri'=>$id]);
        if(count($bankaBilgileri)>0){
            $bankaBilgileri=$bankaBilgileri[0];
        }
        else{
            $bankaBilgileri = new BankaBilgileri();
        }

        $bankaBilgileri->setBanka($banka);
        $bankaBilgileri->setHesapNo($hesapNo);
        $bankaBilgileri->setHesapSahibi($hesapSahibi);
        $bankaBilgileri->setIban($iban);
        $bankaBilgileri->setMusteri($em->getReference(User::class,$id));
        $em->persist($bankaBilgileri);
        $em->flush();



        return new Response('');
    }


    #[Route('/musteriler/faturaBilgileriAc', name: 'app_musteriler_faturaBilgileriAc')]
    public function faturaBilgileriAc(Request $request,EntityManagerInterface $em,RequestStack $session): Response
    {
        $this->dilKontrol();

        $id = $request->request->get('id');


        $qb = $em->createQueryBuilder();
        $veriler=$qb->select('f')
            ->from(FaturaBilgileri::class, 'f')
            ->join('f.musteri','m')
            ->where('m.id='.$id)
            ->getQuery()
            ->getScalarResult();





        return $this->render('panel/musteriler/faturaBilgileri.html.twig',array('veriler'=>$veriler,'id'=>$id));
    }

    #[Route('/musteriler/faturaBilgileriKaydet', name: 'app_musteriler_faturaBilgileriKaydet')]
    public function faturaBilgileriKaydet(Request $request,EntityManagerInterface $em,UserPasswordHasherInterface $passwordEncoder,MailerInterface $mailer, RequestStack $session): Response
    {
        $this->dilKontrol();

        $id = $request->request->get('id');

        $unvan = $request->request->get('unvan');
        $vergiDairesi = $request->request->get('vergiDairesi');
        $vergiNo = $request->request->get('vergiNo');
        $adres = $request->request->get('adres');



        $faturaBilgileri = $em->getRepository(FaturaBilgileri::class)->findBy(['musteri'=>$id]);
        if(count($faturaBilgileri)>0){
            $faturaBilgileri=$faturaBilgileri[0];
        }
        else{
            $faturaBilgileri = new FaturaBilgileri();
        }

        $faturaBilgileri->setUnvan($unvan);
        $faturaBilgileri->setVargiDairesi($vergiDairesi);
        $faturaBilgileri->setVergiNo($vergiNo);
        $faturaBilgileri->setAdres($adres);
        $faturaBilgileri->setMusteri($em->getReference(User::class,$id));
        $em->persist($faturaBilgileri);
        $em->flush();



        return new Response('');
    }


}