<?php

namespace App\Controller;


use App\Entity\Varsayilan;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\Persistence\ManagerRegistry;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\PasswordHasher\Hasher\UserPasswordHasherInterface;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\Security\Http\Authentication\AuthenticationUtils;


class SecurityController extends BaseController
{
    #[Route(path: '/login', name: 'app_login')]
    public function login(AuthenticationUtils $authenticationUtils,RequestStack $requestStack): Response
    {
        $this->dilKontrol();

        if($this->getUser()){
          return $this->redirectToRoute('app_wait');
        }
        // get the login error if there is one
        $error = $authenticationUtils->getLastAuthenticationError();

        // last username entered by the user
        $lastUsername = $authenticationUtils->getLastUsername();

        return $this->render('security/login.html.twig', [
            'last_username' => $lastUsername,
            'error' => $error,
        ]);
    }

    #[Route(path: '/logout', name: 'app_logout')]
    public function logout(): void
    {
        throw new \LogicException('This method can be blank - it will be intercepted by the logout key on your firewall.');
    }

    #[Route('/wait', name: 'app_wait')]
    public function wait(RequestStack $session, EntityManagerInterface $em,ManagerRegistry $doctrine): Response
    {
        $this->dilKontrol();


        if ($this->getUser()->isPassword()==0){
            return $this->redirectToRoute('app_replace_password');
        }



        return $this->redirectToRoute('app_homepage');
    }


    #[Route('/replace-password', name: 'app_replace_password')]
    public function replace_password(Request $request, UserPasswordHasherInterface $passwordEncoder, EntityManagerInterface $em):Response
    {
        $this->dilKontrol();
        $user = $this->getUser();

        if ($user->isPassword()==1){
            return $this->redirectToRoute('swc_homepage');
        }

        $geciciSifre = $request->request->get('geciciSifre');
        $password = $request->request->get('password');
        if ($password){

            if (!$passwordEncoder->isPasswordValid($user,$geciciSifre)){
                return $this->render('security/replace_password.html.twig',array('error'=>'Geçici parolanı yanlış girdin. Tekrar dene.'));
            }

            $user->setPassword($passwordEncoder->hashPassword($user, $password));
            $user->setIsPassword(1);
            $em->persist($user);
            $em->flush();

            return $this->render('security/replace_password.html.twig',array('success'=>'Parola değiştirildi. Yönlendiriliyorsun..'));
        }

        return $this->render('security/replace_password.html.twig');
    }

}
