<?php

namespace App\Controller;

use App\Entity\BankaHesaplari;
use App\Entity\Bankalar;
use App\Entity\Siteler;
use App\Entity\User;
use Doctrine\Persistence\ManagerRegistry;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Mailer\MailerInterface;
use Symfony\Component\PasswordHasher\Hasher\UserPasswordHasherInterface;
use Symfony\Component\Routing\Annotation\Route;

#[Route('/siteler')]
class SitelerController extends BaseController
{

    #[Route('', name: 'app_siteler_homepage')]
    public function index(): Response
    {
        $this->dilKontrol();

        return $this->render('panel/siteler/index.html.twig');
    }

    #[Route('/result', name: 'app_siteler_result')]
    public function result(ManagerRegistry $doctrine,Request $request): Response
    {
        $this->dilKontrol();


        $em =  $doctrine->getManager();


        $draw = $request->request->get('draw');
        $row= $request->request->get('start');
        $rowperpage= $request->request->get('length');
        $searchValue= $request->request->all('search')['value'];



        $sql="s.remove is null";
        if($searchValue != ''){
            $sql .=" and (s.url like '%$searchValue%' or m.adsoyad like '%$searchValue%' or s.komisyon like '%$searchValue%')";
        }


        $qb = $em->createQueryBuilder();
        $hepsi=$qb->select('s.id')
            ->from(Siteler::class, 's')
            ->join('s.musteri','m')
            ->where("s.remove is null")
            ->getQuery()
            ->getScalarResult();
        $totalRecords = count($hepsi);

        $qb = $em->createQueryBuilder();
        $hepsi=$qb->select('s.id')
            ->from(Siteler::class, 's')
            ->join('s.musteri','m')
            ->where($sql)
            ->getQuery()
            ->getScalarResult();

        $totalRecordwithFilter = count($hepsi);
        $qb = $em->createQueryBuilder();
        $veriler=$qb->select('s,m.adsoyad')
            ->from(Siteler::class, 's')
            ->join('s.musteri','m')
            ->where($sql)
            ->setFirstResult($row)
            ->setMaxResults($rowperpage)
            ->getQuery()
            ->getScalarResult();

        $data = array();

        foreach ($veriler as $veri){


            $islem = $this->islemler($veri['s_id']);



          //  ['url','musteri','komisyon','durum','tarih','islem']
            $data[] = array(
                    "url" => '<a href="http://'.$veri['s_url'].'" target="_blank"><span class="text-primary fw-bold">'.$veri['s_url'].'</span></a> ',
                    "musteri" => $veri['adsoyad'],
                    "komisyon" => '% '.$veri['s_komisyon'],
                    "vade" => $veri['s_vade'],
                    "durum" => $veri['s_durum'] ? '<span class="badge bg-label-success">'.$this->ceviri('Aktif').'</span>' : '<span class="badge bg-label-danger">'.$this->ceviri('Pasif').'</span>',
                    "tarih" => $veri['s_olusturmaTarihi']->format('d.m.Y H:i'),
                    "islem" => $islem,
                );

        }

        $response = array(
            "draw" => intval($draw),
            "recordsTotal" => $totalRecords,
            "recordsFiltered" => $totalRecordwithFilter,
            "data" => $data
        );

        return $this->json($response);
    }

    #[Route('/remove', name: 'app_siteler_remove')]
    public function remove(Request $request,ManagerRegistry $doctrine): Response
    {
        $this->dilKontrol();
        $em =  $doctrine->getManager();

        $id = $request->request->get('id');

        $veri = $em->getRepository(Siteler::class)->find($id);
        $veri->setRemove(1);
        $em->persist($veri);
        $em->flush();

        return new Response('');
    }

    #[Route('/modal', name: 'app_siteler_modal')]
    public function modal(Request $request,ManagerRegistry $doctrine): Response
    {
        $this->dilKontrol();
        $em =  $doctrine->getManager();

        $id = $request->request->get('id');

        $veriler=[];
        if($id!="add"){
            $qb = $em->createQueryBuilder();
            $veriler=$qb->select('s,m.id as mId')
                ->from(Siteler::class, 's')
                ->join('s.musteri','m')
                ->where('s.id='.$id)
                ->getQuery()
                ->getScalarResult();
        }




        $qb = $em->createQueryBuilder();
        $musteriler=$qb->select('u.id as value, u.adsoyad as label')
            ->from(User::class, 'u')
            ->where("u.remove is null and u.roles like '%ROLE_MUSTERI%'")
            ->getQuery()
            ->getScalarResult();


        return $this->render('panel/siteler/modal.html.twig',array('veriler'=>$veriler,'musteriler'=>$musteriler,'id'=>$id));
    }
    #[Route('/modalSave', name: 'app_siteler_modalSave')]
    public function modalSave(Request $request,ManagerRegistry $doctrine,UserPasswordHasherInterface $passwordEncoder,MailerInterface $mailer): Response
    {
        $this->dilKontrol();
        $em =  $doctrine->getManager();

        $id = $request->request->get('id');

        $url = $request->request->get('url');
        $musteri = $request->request->get('musteri');
        $komisyon = $request->request->get('komisyon');
        $durum = $request->request->get('durum');
        $vade = $request->request->get('vade');


        if($id!="add"){
            $data = $em->getRepository(Siteler::class)->find($id);
        }else{
            $data = new Siteler();
            $data->setOlusturmaTarihi(new \DateTime('now'));

        }

        $data->setUrl($url);
        $data->setMusteri($em->getReference(User::class, $musteri));
        $data->setKomisyon($komisyon);
        $data->setDurum($durum);
        $data->setVade($vade);
        $em->persist($data);
        $em->flush();


      return new Response('');
    }



}