<?php

namespace App\Entity;

use App\Repository\SitelerRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\DBAL\Types\Types;
use Doctrine\ORM\Mapping as ORM;

#[ORM\Entity(repositoryClass: SitelerRepository::class)]
class Siteler
{
    #[ORM\Id]
    #[ORM\GeneratedValue]
    #[ORM\Column]
    private ?int $id = null;

    #[ORM\Column(length: 255)]
    private ?string $url = null;

    #[ORM\ManyToOne(inversedBy: 'siteler')]
    #[ORM\JoinColumn(nullable: false)]
    private ?User $musteri = null;

    #[ORM\Column]
    private ?float $komisyon = null;

    #[ORM\Column]
    private ?bool $durum = null;

    #[ORM\Column(type: Types::DATETIME_MUTABLE)]
    private ?\DateTimeInterface $olusturmaTarihi = null;

    #[ORM\Column(nullable: true)]
    private ?bool $remove = null;

    /**
     * @var Collection<int, Odemeler>
     */
    #[ORM\OneToMany(targetEntity: Odemeler::class, mappedBy: 'site', orphanRemoval: true)]
    private Collection $odemeler;

    /**
     * @var Collection<int, Raporlar>
     */
    #[ORM\OneToMany(targetEntity: Raporlar::class, mappedBy: 'site')]
    private Collection $raporlar;

    #[ORM\Column(nullable: true)]
    private ?int $vade = null;

    public function __construct()
    {
        $this->odemeler = new ArrayCollection();
        $this->raporlar = new ArrayCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getUrl(): ?string
    {
        return $this->url;
    }

    public function setUrl(string $url): static
    {
        $this->url = $url;

        return $this;
    }

    public function getMusteri(): ?User
    {
        return $this->musteri;
    }

    public function setMusteri(?User $musteri): static
    {
        $this->musteri = $musteri;

        return $this;
    }

    public function getKomisyon(): ?float
    {
        return $this->komisyon;
    }

    public function setKomisyon(float $komisyon): static
    {
        $this->komisyon = $komisyon;

        return $this;
    }

    public function isDurum(): ?bool
    {
        return $this->durum;
    }

    public function setDurum(bool $durum): static
    {
        $this->durum = $durum;

        return $this;
    }

    public function getOlusturmaTarihi(): ?\DateTimeInterface
    {
        return $this->olusturmaTarihi;
    }

    public function setOlusturmaTarihi(\DateTimeInterface $olusturmaTarihi): static
    {
        $this->olusturmaTarihi = $olusturmaTarihi;

        return $this;
    }

    public function isRemove(): ?bool
    {
        return $this->remove;
    }

    public function setRemove(?bool $remove): static
    {
        $this->remove = $remove;

        return $this;
    }

    /**
     * @return Collection<int, Odemeler>
     */
    public function getOdemeler(): Collection
    {
        return $this->odemeler;
    }

    public function addOdemeler(Odemeler $odemeler): static
    {
        if (!$this->odemeler->contains($odemeler)) {
            $this->odemeler->add($odemeler);
            $odemeler->setSite($this);
        }

        return $this;
    }

    public function removeOdemeler(Odemeler $odemeler): static
    {
        if ($this->odemeler->removeElement($odemeler)) {
            // set the owning side to null (unless already changed)
            if ($odemeler->getSite() === $this) {
                $odemeler->setSite(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection<int, Raporlar>
     */
    public function getRaporlar(): Collection
    {
        return $this->raporlar;
    }

    public function addRaporlar(Raporlar $raporlar): static
    {
        if (!$this->raporlar->contains($raporlar)) {
            $this->raporlar->add($raporlar);
            $raporlar->setSite($this);
        }

        return $this;
    }

    public function removeRaporlar(Raporlar $raporlar): static
    {
        if ($this->raporlar->removeElement($raporlar)) {
            // set the owning side to null (unless already changed)
            if ($raporlar->getSite() === $this) {
                $raporlar->setSite(null);
            }
        }

        return $this;
    }

    public function getVade(): ?int
    {
        return $this->vade;
    }

    public function setVade(?int $vade): static
    {
        $this->vade = $vade;

        return $this;
    }
}
