<?php

namespace App\Entity;

use App\Repository\UserRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\DBAL\Types\Types;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Security\Core\User\PasswordAuthenticatedUserInterface;
use Symfony\Component\Security\Core\User\UserInterface;

#[ORM\Entity(repositoryClass: UserRepository::class)]
class User implements UserInterface, PasswordAuthenticatedUserInterface
{
    #[ORM\Id]
    #[ORM\GeneratedValue]
    #[ORM\Column]
    private ?int $id = null;

    #[ORM\Column(length: 180, unique: true)]
    private ?string $email = null;

    #[ORM\Column(type: 'boolean')]
    private $isVerified = false;

    #[ORM\Column]
    private ?bool $isPassword = null;

    #[ORM\Column]
    private array $roles = [];

    /**
     * @var string The hashed password
     */
    #[ORM\Column]
    private ?string $password = null;

    #[ORM\Column(length: 255, nullable: true)]
    private ?string $adsoyad = null;

    #[ORM\Column(length: 255, nullable: true)]
    private ?string $unvan = null;

    #[ORM\Column(length: 255)]
    private ?string $telefon = null;

    #[ORM\Column(length: 255, nullable: true)]
    private ?string $resim = null;

    #[ORM\ManyToOne(inversedBy: 'user')]
    private ?Rol $rol = null;

    #[ORM\Column(nullable: true)]
    private ?bool $remove = null;

    #[ORM\Column(length: 255,nullable: true)]
    private ?string $reset_token = null;

    #[ORM\Column(type: Types::DATETIME_MUTABLE, nullable: true)]
    private ?\DateTimeInterface $lastLogin = null;

    /**
     * @var Collection<int, BankaBilgileri>
     */
    #[ORM\OneToMany(targetEntity: BankaBilgileri::class, mappedBy: 'musteri', orphanRemoval: true)]
    private Collection $bankaBilgileri;

    /**
     * @var Collection<int, FaturaBilgileri>
     */
    #[ORM\OneToMany(targetEntity: FaturaBilgileri::class, mappedBy: 'musteri', orphanRemoval: true)]
    private Collection $faturaBilgileri;

    /**
     * @var Collection<int, Siteler>
     */
    #[ORM\OneToMany(targetEntity: Siteler::class, mappedBy: 'musteri', orphanRemoval: true)]
    private Collection $siteler;

    #[ORM\Column(type: Types::TEXT, nullable: true)]
    private ?string $ekstraEmbed = null;


    public function __construct()
    {
        $this->bankaBilgileri = new ArrayCollection();
        $this->faturaBilgileri = new ArrayCollection();
        $this->siteler = new ArrayCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getEmail(): ?string
    {
        return $this->email;
    }

    public function setEmail(string $email): static
    {
        $this->email = $email;

        return $this;
    }

    /**
     * A visual identifier that represents this user.
     *
     * @see UserInterface
     */
    public function getUserIdentifier(): string
    {
        return (string) $this->email;
    }

    /**
     * @see UserInterface
     */
    public function getRoles(): array
    {
        $roles = $this->roles;
        // guarantee every user at least has ROLE_USER
        $roles[] = 'ROLE_USER';

        return array_unique($roles);
    }

    public function setRoles(array $roles): static
    {
        $this->roles = $roles;

        return $this;
    }

    /**
     * @see PasswordAuthenticatedUserInterface
     */
    public function getPassword(): string
    {
        return $this->password;
    }

    public function setPassword(string $password): static
    {
        $this->password = $password;

        return $this;
    }

    /**
     * @see UserInterface
     */
    public function eraseCredentials(): void
    {
        // If you store any temporary, sensitive data on the user, clear it here
        // $this->plainPassword = null;
    }

    public function getAdsoyad(): ?string
    {
        return $this->adsoyad;
    }

    public function setAdsoyad(?string $adsoyad): static
    {
        $this->adsoyad = $adsoyad;

        return $this;
    }

    public function getUnvan(): ?string
    {
        return $this->unvan;
    }

    public function setUnvan(?string $unvan): static
    {
        $this->unvan = $unvan;

        return $this;
    }

    public function isVerified(): bool
    {
        return $this->isVerified;
    }

    public function setIsVerified(bool $isVerified): static
    {
        $this->isVerified = $isVerified;

        return $this;
    }


    public function isPassword(): ?bool
    {
        return $this->isPassword;
    }

    public function setIsPassword(bool $isPassword): static
    {
        $this->isPassword = $isPassword;

        return $this;
    }

    public function getTelefon(): ?string
    {
        return $this->telefon;
    }

    public function setTelefon(string $telefon): static
    {
        $this->telefon = $telefon;

        return $this;
    }

    public function getResim(): ?string
    {
        return $this->resim;
    }

    public function setResim(?string $resim): static
    {
        $this->resim = $resim;

        return $this;
    }

    public function getRol(): ?Rol
    {
        return $this->rol;
    }

    public function setRol(?Rol $rol): static
    {
        $this->rol = $rol;

        return $this;
    }

    public function isRemove(): ?bool
    {
        return $this->remove;
    }

    public function setRemove(?bool $remove): static
    {
        $this->remove = $remove;

        return $this;
    }

    public function getResetToken(): ?string
    {
        return $this->reset_token;
    }

    public function setResetToken(?string $reset_token): static
    {
        $this->reset_token = $reset_token;

        return $this;
    }

    public function getLastLogin(): ?\DateTimeInterface
    {
        return $this->lastLogin;
    }

    public function setLastLogin(?\DateTimeInterface $lastLogin): static
    {
        $this->lastLogin = $lastLogin;

        return $this;
    }

    /**
     * @return Collection<int, BankaBilgileri>
     */
    public function getBankaBilgileri(): Collection
    {
        return $this->bankaBilgileri;
    }

    public function addBankaBilgileri(BankaBilgileri $bankaBilgileri): static
    {
        if (!$this->bankaBilgileri->contains($bankaBilgileri)) {
            $this->bankaBilgileri->add($bankaBilgileri);
            $bankaBilgileri->setMusteri($this);
        }

        return $this;
    }

    public function removeBankaBilgileri(BankaBilgileri $bankaBilgileri): static
    {
        if ($this->bankaBilgileri->removeElement($bankaBilgileri)) {
            // set the owning side to null (unless already changed)
            if ($bankaBilgileri->getMusteri() === $this) {
                $bankaBilgileri->setMusteri(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection<int, FaturaBilgileri>
     */
    public function getFaturaBilgileri(): Collection
    {
        return $this->faturaBilgileri;
    }

    public function addFaturaBilgileri(FaturaBilgileri $faturaBilgileri): static
    {
        if (!$this->faturaBilgileri->contains($faturaBilgileri)) {
            $this->faturaBilgileri->add($faturaBilgileri);
            $faturaBilgileri->setMusteri($this);
        }

        return $this;
    }

    public function removeFaturaBilgileri(FaturaBilgileri $faturaBilgileri): static
    {
        if ($this->faturaBilgileri->removeElement($faturaBilgileri)) {
            // set the owning side to null (unless already changed)
            if ($faturaBilgileri->getMusteri() === $this) {
                $faturaBilgileri->setMusteri(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection<int, Siteler>
     */
    public function getSiteler(): Collection
    {
        return $this->siteler;
    }

    public function addSiteler(Siteler $siteler): static
    {
        if (!$this->siteler->contains($siteler)) {
            $this->siteler->add($siteler);
            $siteler->setMusteri($this);
        }

        return $this;
    }

    public function removeSiteler(Siteler $siteler): static
    {
        if ($this->siteler->removeElement($siteler)) {
            // set the owning side to null (unless already changed)
            if ($siteler->getMusteri() === $this) {
                $siteler->setMusteri(null);
            }
        }

        return $this;
    }

    public function getEkstraEmbed(): ?string
    {
        return $this->ekstraEmbed;
    }

    public function setEkstraEmbed(?string $ekstraEmbed): static
    {
        $this->ekstraEmbed = $ekstraEmbed;

        return $this;
    }



}
