<?php

// src/Twig/AppExtension.php
namespace App\Twig;

use App\Entity\Adresler;
use App\Entity\Ayarlar;
use App\Entity\Ceviri;
use App\Entity\Dil;
use App\Entity\Fiyatlar;
use App\Entity\Izinler;
use App\Entity\Rol;
use App\Entity\Surumler;
use App\Entity\Urun;
use App\Entity\UrunKategori;
use App\Entity\UrunOzellik;
use App\Entity\User;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\Persistence\ManagerRegistry;
use Symfony\Component\HttpFoundation\RequestStack;
use Twig\Extension\AbstractExtension;
use Twig\TwigFilter;

class AppExtension extends AbstractExtension
{
    private $em;
    private $doctrine;
    private $session;

    public function __construct(RequestStack $session,EntityManagerInterface $em,ManagerRegistry $doctrine) {
        $this->em = $em;
        $this->doctrine = $doctrine;
        $this->session = $session;
    }

    public function getFilters()
    {
        return [
            /*            new TwigFilter('test', [AppRuntime::class, 'test']),*/

            new TwigFilter('uzantiBul', [$this, 'uzantiBul']),
            new TwigFilter('SHA1', [$this, 'SHA1']),
            new TwigFilter('rasgele', [$this, 'rasgele']),
            new TwigFilter('rasgeleSayi', [$this, 'rasgeleSayi']),
            new TwigFilter('seo', [$this, 'seo']),
            new TwigFilter('tarih_yaz', [$this, 'tarih_yaz']),
            new TwigFilter('base64_encode', [$this, 'base64_encode']),
            new TwigFilter('base64_decode', [$this, 'base64_decode']),
            new TwigFilter('yandexToken', [$this, 'yandexToken']),
            new TwigFilter('cevir', [$this, 'cevir']),
            new TwigFilter('ceviri', [$this, 'ceviri']),
            new TwigFilter('GetIP', [$this, 'GetIP']),
            new TwigFilter('get_browser_name', [$this, 'get_browser_name']),
            new TwigFilter('html_entity_decode', [$this, 'html_entity_decode']),
            new TwigFilter('d2ArrayConvert', [$this, 'd2ArrayConvert']),
            new TwigFilter('ayarlar', [$this, 'ayarlar']),
            new TwigFilter('imgOlustur', [$this, 'imgOlustur']),
            new TwigFilter('avatarHazirla', [$this, 'avatarHazirla']),
            new TwigFilter('avatarHazirla2', [$this, 'avatarHazirla2']),
            new TwigFilter('avatarHazirla3', [$this, 'avatarHazirla3']),
            new TwigFilter('rolKullaniciBul', [$this, 'rolKullaniciBul']),
            new TwigFilter('sonSurumBul', [$this, 'sonSurumBul']),
            new TwigFilter('izinBul', [$this, 'izinBul']),
            new TwigFilter('firmaizinBul', [$this, 'firmaizinBul']),
            new TwigFilter('rolDetay', [$this, 'rolDetay']),
            new TwigFilter('modulDetay', [$this, 'modulDetay']),
            new TwigFilter('yetkiBak', [$this, 'yetkiBak']),
            new TwigFilter('kargoTanimiTipleri', [$this, 'kargoTanimiTipleri']),
            new TwigFilter('firmaYetkiliBul', [$this, 'firmaYetkiliBul']),
            new TwigFilter('musteriYetkiliBul', [$this, 'musteriYetkiliBul']),
            new TwigFilter('fiyatDetaylar', [$this, 'fiyatDetaylar']),
            new TwigFilter('adresDetaylar', [$this, 'adresDetaylar']),
            new TwigFilter('anaFiyatGetir', [$this, 'anaFiyatGetir']),
            new TwigFilter('entityDetay', [$this, 'entityDetay']),
            new TwigFilter('sepetSay', [$this, 'sepetSay']),
            new TwigFilter('stokAyarla', [$this, 'stokAyarla']),
            new TwigFilter('anaFiyatGetir2', [$this, 'anaFiyatGetir2']),
            new TwigFilter('urun_altKategoriGeriYazBreadcrumb', [$this, 'urun_altKategoriGeriYazBreadcrumb']),
            new TwigFilter('urunAltKategori', [$this, 'urunAltKategori']),
            new TwigFilter('urunAltKategori2', [$this, 'urunAltKategori2']),
            new TwigFilter('urunAltOzellik2', [$this, 'urunAltOzellik2']),
            new TwigFilter('musteriTipleri', [$this, 'musteriTipleri']),
            new TwigFilter('karMarji', [$this, 'karMarji']),
            new TwigFilter('urunDetay', [$this, 'urunDetay']),
            new TwigFilter('fiyatDetay', [$this, 'fiyatDetay']),
            new TwigFilter('islemler', [$this, 'islemler']),
            new TwigFilter('urunAltOzellik', [$this, 'urunAltOzellik']),
            new TwigFilter('urun_altKategoriId', [$this, 'urun_altKategoriId']),
            new TwigFilter('altGruplar', [$this, 'altGruplar']),
            new TwigFilter('ozellikBul', [$this, 'ozellikBul']),
            new TwigFilter('getCombinations', [$this, 'getCombinations']),
            new TwigFilter('kullaniciDetaylar', [$this, 'kullaniciDetaylar']),
            new TwigFilter('compareLines', [$this, 'compareLines']),


        ];
    }

    function ozellikBul($id,$selected="",$kendisi=0){
        $em = $this->em;

        $xx = $this->entityDetay('UrunOzellik','secim',$id);
        $type = 'radio';
        if($xx==1){$type='checkbox';}

        $qb = $em->createQueryBuilder();
        $ozellikler=$qb->select('k.id,k.adi')
            ->from(UrunOzellik::class, 'k')
            ->where('k.ustid='.$id)
            ->orderBy('k.sira','ASC')
            ->orderBy('k.id','ASC')
            ->getQuery()
            ->getScalarResult();

        $veriler='';
        if ($selected!=""){
            $gelenOzellikler = explode(',',$selected);
        }

        foreach ($ozellikler as $k){
            $checked='';
            if ($selected!="") {
                if (in_array($k['id'], $gelenOzellikler)) {
                    $checked = 'checked';
                }
            }
            $disabled='';
            if($kendisi==$k['id']){
                $disabled = 'disabled';
            }
            $veriler.=' <div class="col-md-6">
                            <div class="form-group row">
                                <label for="example-text-input" class="col-sm-6 col-form-label">'.$k['adi'].' :</label>
                                <div class="col-sm-6" style="padding-top: 8px;">
                                  
                                      <label class="switch switch-square switch-lg ms-3">
                                        <input type="'.$type.'" id="ozellik'.$k['id'].'" class="switch-input ozellik" '.$checked.' name="ozellik'.$id.'"  value="'.$k['id'].'">
                                        <span class="switch-toggle-slider">
                                            <span class="switch-on"><i class="ti ti-check"></i></span>
                                            <span class="switch-off"><i class="ti ti-x"></i></span>
                                        </span>
                                        <span class="switch-label"></span>
                                    </label>
                                </div>
                            </div>
                        </div>';
        }

        return $veriler;


    }

    public function altGruplar($id){
        $em = $this->em;

        $qb = $em->createQueryBuilder();
        $ozellikler=$qb->select('k.id,k.adi')
            ->from(UrunOzellik::class, 'k')
            ->where('k.ustid='.$id)
            ->orderBy('k.sira','ASC')
            ->getQuery()
            ->getScalarResult();

        return $ozellikler;
    }


    function urun_altKategoriId($id){
        $em = $this->em;

        $qb = $em->createQueryBuilder();
        $kategoriler=$qb->select('k.id')
            ->from(UrunKategori::class, 'k')
            ->where('k.ustid='.$id)
            ->orderBy('k.sira','ASC')
            ->getQuery()
            ->getScalarResult();


        $veriler='';
        if(count($kategoriler)>0) {

            foreach ($kategoriler as $k) {

                $veriler .= $k['id'] . ',';
                $veriler .= $this->urun_altKategoriId($k['id']);


            }
        }

        return $veriler;


    }

    public function islemler($id,$yenileme='false',$tip=0,$icSayfa='')
    {

        $session = $this->session;

        if($icSayfa==''){
            $duzenle = ' <li><a class="dropdown-item" href="javascript:void(0);" onclick="modal(' . $id . ','.$yenileme.','.$tip.');"><i class="ti ti-edit"></i> Düzenle</a></li>';
        }else{
            $duzenle = ' <li><a class="dropdown-item" href="'.$icSayfa.$id.'"><i class="ti ti-edit"></i> Düzenle</a></li>';
        }

        $sil = '<li><a class="dropdown-item" href="javascript:void(0);" id="sil_'.$id.'" tip="'.$tip.'"  onclick="sil(' . $id . ');"><i class="ti ti-trash"></i> Sil</a></li>';

        if($session->getSession()->get('superApp_izin')==1 or ($session->getSession()->get('pageIzin')['duzenleme']??"") == 1){}else{
            $duzenle = '';
        }

        if($session->getSession()->get('superApp_izin')==1 or ($session->getSession()->get('pageIzin')['olusturma']??"") == 1){}else{
            $sil = '';
        }


        return '<div class="btn-group">

                          <button type="button" class="btn btn-light btn-icon rounded-pill dropdown-toggle hide-arrow waves-effect waves-light" data-bs-toggle="dropdown" aria-expanded="false">
                            <i class="ti ti-dots-vertical"></i>
                          </button>
                          <ul class="dropdown-menu dropdown-menu-end" style="">
                           '.$duzenle.'
                            <li>
                              <hr class="dropdown-divider">
                            </li>
                            '.$sil.'
                          </ul>
                        </div>';
    }


    function uzantiBul($type){
        if($type=='image/jpeg'){ return '.jpg'; }
        elseif($type=='image/png'){ return '.png'; }
        elseif($type=='image/gif'){ return '.gif'; }
        elseif($type=='video/mp4'){ return '.mp4'; }
        elseif($type=='application/rtf'){ return '.rtf'; }
        elseif($type=='text/plain'){ return '.txt'; }
        elseif($type=='application/vnd.ms-powerpoint'){ return '.ppt'; }
        elseif($type=='application/vnd.openxmlformats-officedocument.presentationml.presentation'){ return '.pptx'; }
        elseif($type=='application/xml'){ return '.xml'; }
        elseif($type=='text/xml'){ return '.xml'; }
        elseif($type=='application/vnd.ms-excel'){ return '.xls'; }
        elseif($type=='application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'){ return '.xlsx'; }
        elseif($type=='application/msword'){ return '.doc'; }
        elseif($type=='application/vnd.openxmlformats-officedocument.wordprocessingml.document'){ return '.docx'; }
        else{ return '.belirsiz'; }


    }


    public function compareLines($text1, $text2)
    {
        $lines1 = explode("\n", $text1);
        $lines2 = explode("\n", $text2);
        $maxLines = max(count($lines1), count($lines2));

        $result = [];

        for ($i = 0; $i < $maxLines; $i++) {
            $line1 = $lines1[$i] ?? '';
            $line2 = $lines2[$i] ?? '';
            $result[] = trim($line1) === trim($line2);
        }

        return $result;
    }

    public function html_entity_decode($degisken){
        $degisken = html_entity_decode($degisken, ENT_QUOTES,"UTF-8");
        return $degisken;
    }
    public function seo($url)
    {

        $tr = array('ş','Ş','ı','I','İ','ğ','Ğ','ü','Ü','ö','Ö','Ç','ç','(',')','/',':',',');
        $eng = array('s','s','i','i','i','g','g','u','u','o','o','c','c','','','-','-','');
        $s = str_replace($tr,$eng,$url);

        $find = array('/[^A-Za-z0-9\-<>]/', '/[\-]+/', '/<[^>]*>/');
        $repl = array('-', '-', '-');
        $s = preg_replace($find, $repl, $s);

        $s = strtolower($s);
        $s = preg_replace('/&amp;amp;amp;amp;amp;amp;amp;amp;amp;.+?;/', '', $s);
        $s = preg_replace('/\s+/', '-', $s);
        $s = preg_replace('|-+|', '-', $s);
        $s = preg_replace('/#/', '', $s);
        $s = str_replace('.', '', $s);
        $s = trim($s, '-');
        $s = strtolower($s);
        return $s;

    }
    public function tarih_yaz($tarih){
        $aylar = array('January','February','March','April','May','June','July','August','September','October','November','December','Sunday','Monday','Tuesday','Wednesday','Thursday','Friday','Saturday');
        $aylar2 = array('Ocak','Şubat','Mart','Nisan','Mayıs','Haziran','Temmuz','Ağustos','Eylül','Ekim','Kasım','Aralık','Pazar','Pazartesi','Salı','Çarşamba','Perşembe','Cuma','Cumartesi');
        $return = str_replace($aylar, $aylar2, $tarih);

        return $return;
    }
    public function base64_encode($veri){

        return base64_encode($veri);
    }
    public function base64_decode($veri){

        return base64_decode($veri);
    }
    public function GetIP()
    {
        if (getenv("HTTP_CLIENT_IP")) {
            $ip = getenv("HTTP_CLIENT_IP");
        } elseif (getenv("HTTP_X_FORWARDED_FOR")) {
            $ip = getenv("HTTP_X_FORWARDED_FOR");
            if (strstr($ip, ',')) {
                $tmp = explode(',', $ip);
                $ip = trim($tmp[0]);
            }
        } else {
            $ip = getenv("REMOTE_ADDR");
        }
        return $ip;
    }
    public function get_browser_name($user_agent)
    {
        if (strpos($user_agent, 'Opera') || strpos($user_agent, 'OPR/')) return 'Opera';
        elseif (strpos($user_agent, 'Edge')) return 'Edge';
        elseif (strpos($user_agent, 'Chrome')) return 'Chrome';
        elseif (strpos($user_agent, 'Safari')) return 'Safari';
        elseif (strpos($user_agent, 'Firefox')) return 'Firefox';
        elseif (strpos($user_agent, 'MSIE') || strpos($user_agent, 'Trident/7')) return 'İnternet Explorer';

        return 'Diğer';
    }
    function d2ArrayConvert($array){

        $array  = unserialize($array);

        return $array;
    }
    function rasgele($uzunluk)
    {
        $key='';
        $karakterler = "1234567890ABCDEFGHIJKLMNOPQRSTUVWXYZ";
        for($i=0;$i<$uzunluk;$i++)
        {
            $key .= $karakterler[rand(0,35)];
        }
        return $key;
    }
    function rasgeleSayi($uzunluk)
    {
        $key='';
        $karakterler = "1234567890";
        for($i=0;$i<$uzunluk;$i++)
        {
            $key .= $karakterler[rand(0,9)];
        }
        return $key;
    }
    function SHA1($key){
        return sha1($key);
    }

    public function ayarlar($alan)
    {

        $em = $this->em;

        $qb = $em->createQueryBuilder();
        $veriler=$qb->select('a.'.$alan)
            ->from(Ayarlar::class, 'a')
            ->where('a.id=1')
            ->getQuery()
            ->getScalarResult();

        return $veriler[0][$alan];

    }

    public function imgOlustur($resim,$yukseklik=300,$radius=0,$size='cover'){
        if ($resim==''){
            $resim="resimyok.webp";
        }
        return '<div style="background-image: url(/uploads/'.$resim.');background-size: '.$size.'; background-repeat: no-repeat; overflow: hidden; width: 100%; height: '.$yukseklik.'px; background-position: center;border-radius: '.$radius.'px;"></div>';
    }

    public function avatarHazirla($resim,$adsoyad,$rYukseklik){

        if($resim==''){
            $data = '<span class="avatar-initial rounded-circle bg-label-primary">'.mb_substr(explode(' ',$adsoyad)[0],0,1,'UTF-8').mb_substr(explode(' ',$adsoyad)[1]??"",0,1,'UTF-8').'</span>';
        }else{
            $data = $this->imgOlustur($resim,$rYukseklik,100);
        }


        return $data;

    }
    public function avatarHazirla2($resim,$isim,$rYukseklik){



        if($resim==''){
            $data = '<div class="avatar avatar-md me-3"><span class="avatar-initial bg-label-primary" style="border-radius: 10px;">'.mb_substr(explode(' ',$isim)[0],0,1,'UTF-8').mb_substr(explode(' ',$isim)[1]??"",0,1,'UTF-8').'</span></div>';
        }else{
            $data = '<div class="avatar avatar-md me-3"><span class="avatar-initial bg-label-primary p-1" style="border-radius: 10px;">'.$this->imgOlustur($resim,48,10,'contain').'</span></div>';
        }

        return $data;

    }
    public function avatarHazirla3($resim,$isim,$rYukseklik){



        if($resim==''){
            $data = '<div class="avatar avatar-md me-3"><span class="avatar-initial rounded-circle bg-label-primary">'.mb_substr(explode(' ',$isim)[0],0,1,'UTF-8').mb_substr(explode(' ',$isim)[1]??"",0,1,'UTF-8').'</span></div>';
        }else{
            $data = '<div style="width: 120px; height: 48px; margin-right: 30px;">'.$this->imgOlustur($resim,48,100,'contain').'</div>';
        }

        return $data;

    }
    public function rolKullaniciBul($rol){
        $session = $this->session;
        $ffSql = $session->getSession()->get('firma')==null ? " and u.firma is null" : " and u.firma = ".$session->getSession()->get('firma');

        $em = $this->em;

        $qb = $em->createQueryBuilder();
        $veriler=$qb->select('u.resim,u.adsoyad')
            ->from(User::class, 'u')
            ->join('u.rol', 'r')
            ->where('r.id='.$rol)
            ->getQuery()
            ->getScalarResult();



        return $veriler;
    }
    public function sonSurumBul(){

        $em = $this->em;

        $qb = $em->createQueryBuilder();
        $veriler=$qb->select('s.versionNumarasi')
            ->from(Surumler::class, 's')
            ->where('s.remove is null and s.durum=1')
            ->orderBy('s.tarih','DESC')
            ->setMaxResults(1)
            ->getQuery()
            ->getScalarResult();


        return count($veriler)>0 ? $veriler[0]['versionNumarasi'] : 0;
    }
    public function rolDetay($id,$alan){
        $em = $this->em;

        $qb = $em->createQueryBuilder();
        $modul=$qb->select('m.'.$alan)
            ->from(Rol::class, 'm')
            ->where('m.id='.$id)
            ->getQuery()
            ->getScalarResult();

        return $modul[0][$alan] ?? "";

    }
    public function modulDetay($id,$alan){
        $em = $this->em;

        $qb = $em->createQueryBuilder();
        $modul=$qb->select('m.'.$alan)
            ->from(Izinler::class, 'm')
            ->where('m.id='.$id)
            ->getQuery()
            ->getScalarResult();

        return $modul[0][$alan] ?? "";

    }


    public function izinBul($rol)
    {
        $em = $this->em;
        $izinler = [];

        if($rol!='') {
            $qb = $em->createQueryBuilder();
            $rol = $qb->select('s.izinler')
                ->from(Rol::class, 's')
                ->where('s.id=' . $rol)
                ->getQuery()
                ->getScalarResult();


            foreach ($this->d2ArrayConvert($rol[0]['izinler']) as $key => $izin) {
                $dd = $izin;
                $dd['route'] = $this->modulDetay($key, 'route');
                $izinler[$key] = $dd;
            }
        }

        return $izinler;
    }
    public function firmaizinBul($fId)
    {
        $em = $this->em;
        $izinler = [];

        if($fId!='') {
            $qb = $em->createQueryBuilder();
            $rol = $qb->select('s.izinler')
                ->from(Firmalar::class, 's')
                ->where('s.id=' . $fId)
                ->getQuery()
                ->getScalarResult();


            foreach ($this->d2ArrayConvert($rol[0]['izinler']) as $key => $izin) {
                $dd = $izin;
                $dd['route'] = $this->modulDetay($key, 'route');
                $izinler[$key] = $dd;
            }
        }

        return $izinler;
    }

    public function yetkiBak($route,$rol,$izinler)
    {
        if($rol == 'ROLE_DEV'){
            if(strstr($route,'/firma/')){
                return 0;
            }
            return 1;
        }else{

            foreach ($izinler as $izin){
                if($izin['route']==$route and $izin['goruntuleme']==1){
                    return 1;
                }
            }
        }

        return 0;
    }

    public function kargoTanimiTipleri($id='')
    {
        $data = ['Kargo Tipleri','Alım Şekilleri','Teslimat Şekilleri','Teslimat Modelleri','Ulaştırma Türleri','Taşınması Yasak Kargolar'];

        if($id!=''){
            return $data[$id];
        }
        return $data;

    }


    public function firmaYetkiliBul($id)
    {
        $em = $this->em;
        if($id!=''){

            $qb = $em->createQueryBuilder();
            $user = $qb->select('s.adsoyad,s.email,s.telefon,s.unvan')
                ->from(User::class, 's')
                ->join('s.firma','f')
                ->where("s.roles like '%ROLE_SUPERFIRMA%' and f.id=".$id)
                ->getQuery()
                ->getScalarResult();

            return $user[0];
        }

        return "";

    }


    public function musteriYetkiliBul($id)
    {
        $em = $this->em;
        $session = $this->session;
        if($id!=''){

            $qb = $em->createQueryBuilder();
            $user = $qb->select('s.adsoyad,s.email,s.telefon,s.unvan,r.id as rId')
                ->from(User::class, 's')
                ->leftJoin('s.rol','r')
                ->join('s.firma','f')
                ->where("s.roles like '%ROLE_SUPERMUSTERI%' and s.musteriId=".$id." and f.id=".$session->getSession()->get('firma'))
                ->getQuery()
                ->getScalarResult();

            return $user[0];
        }

        return "";

    }

    function fiyatDetaylar($id,$tanim,$varyant=''){

        $session = $this->session;
        $em =  $this->doctrine->getManager($session->getSession()->get('db'));

        $sql = '';
        if($varyant!=''){
            $sql = " and f.varyantlar='$varyant'";
        }



        $qb = $em->createQueryBuilder();
        $fiyatlar=$qb->select('f,t')
            ->from(Fiyatlar::class, 'f')
            ->join('f.tanim','t')
            ->where('f.remove is null and t.id='.$tanim.' and f.urun='.$id.$sql)
            ->getQuery()
            ->getScalarResult();


        return $fiyatlar;

    }
    function adresDetaylar($id){

        $session = $this->session;
        $em =  $this->doctrine->getManager();




        $qb = $em->createQueryBuilder();
        $fiyatlar=$qb->select('f')
            ->from(Adresler::class, 'f')
            ->where('f.remove is null and f.musteri='.$id)
            ->getQuery()
            ->getScalarResult();


        return $fiyatlar;

    }
    function kullaniciDetaylar($id){

        $session = $this->session;
        $em =  $this->doctrine->getManager();


        $qb = $em->createQueryBuilder();
        $fiyatlar=$qb->select('f')
            ->from(User::class, 'f')
            ->where('f.remove is null and f.musteri='.$id)
            ->getQuery()
            ->getScalarResult();


        return $fiyatlar;

    }

    function anaFiyatGetir($id,$tip,$tanim){

        $session = $this->session;
        $em =  $this->doctrine->getManager($session->getSession()->get('db'));


        $sql = '';
        if($tip!=''){
            $sql .= ' and f.'.$tip.'=1';
        }

        if($tanim==1) {
            $firma = $em->getRepository('App\Entity\\' . ucfirst($session->getSession()->get('db')) . '\Musteriler')->find($session->getSession()->get('musteri'));
            $sql.= ' and f.tanim='.$firma->getFiyatTanimi()->getId();
        }


        $qb = $em->createQueryBuilder();
        $fiyatlar=$qb->select('f,t')
            ->from('App\Entity\\'.ucfirst($session->getSession()->get('db')).'\Fiyatlar', 'f')
            ->join('f.tanim','t')
            ->where('f.remove is null and f.aktif=1 and f.urun='.$id.$sql)
            ->getQuery()
            ->getScalarResult();


        return $fiyatlar;

    }
    function anaFiyatGetir2($id,$birimId,$tanim){

        $session = $this->session;
        $em =  $this->doctrine->getManager($session->getSession()->get('db'));

        $sql = '';
        if($tanim==1) {
            $firma = $em->getRepository('App\Entity\\' . ucfirst($session->getSession()->get('db')) . '\Musteriler')->find($session->getSession()->get('musteri'));
            $sql.= ' and f.tanim='.$firma->getFiyatTanimi()->getId();
        }

        $qb = $em->createQueryBuilder();
        $fiyatlar=$qb->select('f,t')
            ->from('App\Entity\\'.ucfirst($session->getSession()->get('db')).'\Fiyatlar', 'f')
            ->join('f.tanim','t')
            ->where('f.remove is null and f.aktif=1 and f.urun='.$id.' and f.birimId='.$birimId.$sql)
            ->getQuery()
            ->getScalarResult();


        return $fiyatlar;

    }

    public function entityDetay($entity,$alan,$id,$sql='')
    {
        $ySql = '';
        if($sql!=''){
            $ySql = ' and '.$sql;
        }else{
            $ySql = ' and m.id = '.$id;
        }

        $doctrine = $this->doctrine;
        $em =  $doctrine->getManager();

        $qb = $em->createQueryBuilder();
        $veri=$qb->select('m.'.$alan)
            ->from('App\Entity\\'.$entity, 'm')
            ->where('m.remove is null'.$ySql)
            ->getQuery()
            ->getScalarResult();

        return $veri[0][$alan]??"";

    }


    public function sepetSay($id)
    {
        $session = $this->session;
        $em =  $this->doctrine->getManager($session->getSession()->get('db'));

        $qb = $em->createQueryBuilder();
        $veri=$qb->select('s.id')
            ->from('App\Entity\\'.ucfirst($session->getSession()->get('db')).'\Sepet', 's')
            ->where('s.musteriId='.$id)
            ->getQuery()
            ->getScalarResult();

        return count($veri);

    }

    public function stokAyarla($stok,$birim)
    {
        $session = $this->session;
        $em =  $this->doctrine->getManager($session->getSession()->get('db'));

        $qb = $em->createQueryBuilder();
        $varsayilan=$qb->select('f.stokGosterimi,f.stokYokText')
            ->from('App\Entity\\'.ucfirst($session->getSession()->get('db')).'\Varsayilan', 'f')
            ->getQuery()
            ->getScalarResult();


        $yStok = '';
        if($stok==''){
            $yStok = '100+ '.$birim;
        }
        elseif($stok==0){
            $yStok = $varsayilan[0]['stokYokText']=='' ? 'Tükenmiş' : $varsayilan[0]['stokYokText'];
        }
        elseif($stok>100){
            $yStok = '100+ '.$birim;
        }
        elseif($stok>50){
            $yStok = '50+ '.$birim;
        }
        elseif($stok>10){
            $yStok = '10+ '.$birim;
        }
        elseif($stok>10){
            $yStok = 'Son '.$stok;
        }


        if($varsayilan[0]['stokGosterimi']==0 or $varsayilan[0]['stokGosterimi']==''){
            if ($stok==''){
                return "Var";
            }elseif ($stok==0){
                return "Yok";
            }else{
                return "Var";
            }
        }

        elseif($varsayilan[0]['stokGosterimi']==1){
            return $yStok;
        }

        elseif($varsayilan[0]['stokGosterimi']==2){
            return $stok.' '.$birim;
        }




    }

    function urun_altKategoriGeriYazBreadcrumb($id,$veriler=[]){

        $session = $this->session;
        $em =  $this->doctrine->getManager($session->getSession()->get('db'));





        $qb = $em->createQueryBuilder();
        $kategoriler=$qb->select('k.id,k.ustid,k.adi')
            ->from(UrunKategori::class, 'k')
            ->where('k.remove is null and k.aktif=1 and k.id='.$id)
            ->orderBy('k.sira', 'ASC')
            ->getQuery()
            ->getScalarResult();


        foreach ($kategoriler as $k){
            $veriler[] = '<li class="breadcrumb-item"><a href="javascript:void(0);"  onclick="filtrele('.$k['id'].');">'.$k['adi'].'</a></li>';
            if ($k['ustid']!=''){
                $veriler = $this->urun_altKategoriGeriYazBreadcrumb($k['ustid'],$veriler);
            }
        }

        return $veriler;

    }



    function urunAltKategori($dilgrup,$veriler=''){

        $session = $this->session;
        $em =  $this->doctrine->getManager($session->getSession()->get('db'));




        $qb = $em->createQueryBuilder();
        $kategoriler=$qb->select('s.id as value,s.adi as label')
            ->from(UrunKategori::class, 's')
            ->where('s.remove is null  and s.aktif=1 and s.ustid='.$dilgrup)
            ->orderBy('s.sira', 'ASC')
            ->getQuery()
            ->getScalarResult();





        if (count($kategoriler)>0){
            $veriler .= '<div class="accordion accordion-flush" id="accordionBas_'.$dilgrup.'">';
                foreach ($kategoriler as $key=>$k){
                    $veriler .= '<div class="accordion-item">';


                        $veriler .= '<h2 class="accordion-header" id="accordionBasH_' . $dilgrup . '_' . $key . '">
                                                    <button class="accordion-button collapsed p-1 pb-2 pt-2" type="button" data-bs-toggle="collapse"  data-bs-target="#accordionBas_' . $dilgrup . '_' . $key . '" aria-expanded="false" aria-controls="accordionBas_' . $dilgrup . '_' . $key . '" onclick=" $(\'.filtreler #kategori\').val(' . $k['value'] . '); filtreUygula();" value="' . $k['value'] . '">
                                                      ' . $k['label'] . '
                                                    </button>
                                                      <hr class="p-0 m-0">
                                                </h2>
                                                <div id="accordionBas_' . $dilgrup . '_' . $key . '" class="accordion-collapse collapse" aria-labelledby="accordionBasH_' . $dilgrup . '_' . $key . '" data-bs-parent="#accordionBas_' . $dilgrup . '">
                                                    <div class="accordion-body p-0 ps-2">';


                        $veriler = $this->urunAltKategori($k['value'],$veriler);
                        $veriler .= '</div></div>';


                    $veriler .= '</div>';
                }

            $veriler .= '</div>';
        }

        return $veriler;


    }

    function renkler($key)
    {
        $renkler = ['FFFFFF','FCFCFC','F9F9F9','F6F6F6','F3F3F3','F0F0F0','EDEDED','EAEAEA','E7E7E7','E4E4E4'];

        return $renkler[$key];
    }
    function urunAltKategori2($dilgrup,$margin,$renk,$veriler=''){

        $session = $this->session;
        $em =  $this->doctrine->getManager($session->getSession()->get('db'));




        $qb = $em->createQueryBuilder();
        $kategoriler=$qb->select('s.id as value,s.adi as label,s.resim')
            ->from(UrunKategori::class, 's')
            ->where('s.remove is null and s.aktif=1 and s.ustid='.$dilgrup)
            ->orderBy('s.sira', 'ASC')
            ->getQuery()
            ->getScalarResult();





        if (count($kategoriler)>0){
            $veriler .= '<div class="collapse colKategori_'.$dilgrup.'" id="kategori_'.$dilgrup.'">';
                foreach ($kategoriler as $key=>$k){

                    $veriler .= '<div class="card" style="padding-left: '.$margin.'px!important; background-color:#'.$this->renkler($renk).';">
                                 <div class="card-body p-2 d-flex align-items-center"  value="' . $k['value'] . '" style="cursor: pointer;"   >
                                  <div class="py-3" style="padding-left: 10px;padding-right: 10px;" data-bs-toggle="collapse" href="#kategori_' . $k['value'] . '"  aria-expanded="false" aria-controls="kategori_' . $k['value'] . '">
                                     <i class="arrowKat fa fa-chevron-right"  style="display: none;" href="#kategori_' . $k['value'] . '"></i>
                                 </div>
                                 <div class="d-flex align-items-center w-100" onclick="getir(' . $k['value'] . ',\''.$this->html_entity_decode($k['label']).'\');$(\'.kategories .card\').removeClass(\'active\'); $(this).parent(\'div\').parent(\'div\').addClass(\'active\');">
                                     '.$this->avatarHazirla2($k['resim'],$this->html_entity_decode($k['label']),38).' 
                                       '.$this->html_entity_decode($k['label']).'
                                  </div>
                                </div>
                              </div>';



                        $veriler = $this->urunAltKategori2($k['value'],($margin+40),($renk+1),$veriler);



                }

            $veriler .= '</div>';
        }

        return $veriler;


    }

    function urunAltOzellik2($dilgrup,$margin,$renk,$veriler=''){

        $session = $this->session;
        $em =  $this->doctrine->getManager($session->getSession()->get('db'));




        $qb = $em->createQueryBuilder();
        $kategoriler=$qb->select('s.id as value,s.adi as label,s.resimVal,s.tip')
            ->from(UrunOzellik::class, 's')
            ->where('s.remove is null and s.tip != 2 and s.ustid='.$dilgrup)
            ->orderBy('s.sira', 'ASC')
            ->getQuery()
            ->getScalarResult();





        if (count($kategoriler)>0){

            $veriler .= '<div class="collapse colKategori_'.$dilgrup.'" id="kategori_'.$dilgrup.'">';
                foreach ($kategoriler as $key=>$k){
                    $veriler .= '<div class="card" style="padding-left: '.$margin.'px!important; background-color:#'.$this->renkler($renk).';">
                                 <div class="card-body p-2 d-flex align-items-center"  value="' . $k['value'] . '" style="cursor: pointer;"   >
                                  <div class="py-3" style="padding-left: 10px;padding-right: 10px;" data-bs-toggle="collapse" href="#kategori_' . $k['value'] . '"  aria-expanded="false" aria-controls="kategori_' . $k['value'] . '">
                                     <i class="arrowKat fa fa-chevron-right"  style="display: none;" href="#kategori_' . $k['value'] . '"></i>
                                 </div>
                                 <div class="d-flex align-items-center w-100" onclick="getir(' . $k['value'] . ',\''.$this->html_entity_decode($k['label']).'\');$(\'.kategories .card\').removeClass(\'active\'); $(this).parent(\'div\').parent(\'div\').addClass(\'active\');">
                                     '.$this->avatarHazirla2($k['resimVal'],$this->html_entity_decode($k['label']),38).' 
                                       '.$this->html_entity_decode($k['label']).'
                                  </div>
                                </div>
                              </div>';



                        $veriler = $this->urunAltOzellik2($k['value'],($margin+40),($renk+1),$veriler);



                }

            $veriler .= '</div>';
        }

        return $veriler;


    }

    function musteriTipleri($id="")
    {
        $data = [
            ['value' => 0,'label'=>'Gerçek'],
            ['value' => 1,'label'=>'Tüzel'],
            ['value' => 2,'label'=>'Yabancı'],
        ];

        if($id==""){
            return $data;
        }else{
            $yData = ['value'=>'','label'=>''];
            foreach ($data as $dd){
                if($dd['value']==$id){
                    $yData = $dd;
                }
            }
            return $yData;
        }

    }

    public function karMarji($fiyat,$karMarji)
    {
        if($karMarji=='' or $karMarji==0){
            return $fiyat;
        }else{
            return $fiyat+(($fiyat*$karMarji)/100);
        }
    }

    public function urunDetay($id)
    {
        $session = $this->session;
        $em =  $this->doctrine->getManager($session->getSession()->get('db'));




        $qb = $em->createQueryBuilder();
        $urun=$qb->select('u.id,u.resimler,u.adi,u.urunKodu,u.stok')
            ->from(Urun::class, 'u')
            ->where('u.remove is null and u.id='.$id)
            ->getQuery()
            ->getScalarResult();


        return $urun[0]??[];
    }
    public function fiyatDetay($id)
    {
        $session = $this->session;
        $em =  $this->doctrine->getManager($session->getSession()->get('db'));


        $qb = $em->createQueryBuilder();
        $urun=$qb->select('f')
            ->from(Fiyatlar::class, 'f')
            ->where('f.remove is null and f.id='.$id)
            ->getQuery()
            ->getScalarResult();


        return $urun[0]??[];
    }


    function urunAltOzellik($dilgrup,$margin,$veriler=''){

        $session = $this->session;
        $em =  $this->doctrine->getManager($session->getSession()->get('db'));




        $qb = $em->createQueryBuilder();
        $ozellikler=$qb->select('s.id,s.adi,s.ustid,s.kategori,s.tip')
            ->from(UrunOzellik::class, 's')
            ->where('s.remove is null and s.ustid='.$dilgrup)
            ->orderBy('s.sira', 'ASC')
            ->getQuery()
            ->getScalarResult();





        if (count($ozellikler)>0){
            $veriler .= '<div class="collapse colKategori_'.$dilgrup.'" id="kategori_'.$dilgrup.'">';
            foreach ($ozellikler as $key=>$k){
                if($k['tip']==1){
                    $color= 'background-color: #f0fff7;';
                }else{
                    $color= 'background-color: #f3f2ff;';
                }

                $veriler .= '<div class="card mb-1" style="margin-left: '.$margin.'px!important;  '.$color.'">
                                 <div class="card-body p-2 d-flex align-items-center"  value="' . $k['id'] . '" style="cursor: pointer;"   >
                                  <div class="py-3" style="padding-left: 10px;padding-right: 10px;" data-bs-toggle="collapse" href="#kategori_' . $k['id'] . '"  aria-expanded="false" aria-controls="kategori_' . $k['id'] . '">
                                     <i class="arrowKat fa fa-chevron-right"  style="display: none;" href="#kategori_' . $k['id'] . '"></i>
                                 </div>
                                 <div class="d-flex align-items-center w-100 justify-content-between" onclick="">
                                  
                                       '.$this->html_entity_decode($k['adi']).'
                                       <div class="d-flex">
                                       <a type="button" class="btn rounded-pill btn-icon btn-label-success waves-effect waves-light me-1 handle'.$dilgrup.'">
                                              <span class="fa fa-arrows"></span>
                                          </a>
                                       '.$this->islemler($k['id']).'
                                       </div>
                                  </div>
                                </div>
                              </div>';



                $veriler = $this->urunAltOzellik($k['id'],($margin+40),$veriler);



            }

            $veriler .= '</div>';
        }

        return $veriler;


    }



    function getCombinations($arrays) {
        $result = [[]];

        foreach ($arrays as $property) {
            $tmp = [];

            foreach ($result as $resultItem) {
                foreach ($property['vv']['alt'] as $item) {
                    $newCombination = $resultItem;

                    $newCombination[] = [
                        'property' => [
                            'value' => $property['vv']['value'],
                            'label' => $property['vv']['label'],
                        ],
                        'value' => $item['value'],
                        'label' => $item['label']
                    ];

                    $tmp[] = $newCombination;
                }
            }

            $result = $tmp;
        }

        return $result;
    }




    function yandexToken(){
        $curl = curl_init();

        curl_setopt_array($curl, array(
            CURLOPT_URL => 'https://iam.api.cloud.yandex.net/iam/v1/tokens',
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'POST',
            CURLOPT_POSTFIELDS =>'{
    "yandexPassportOauthToken":"y0_AgAAAAAZYEETAATuwQAAAADuaUPsZ3x4KXscRqel7NbnsyEYPaf2HWE"
}',
            CURLOPT_HTTPHEADER => array(
                'Content-Type: application/json'
            ),
        ));

        $response = curl_exec($curl);
        $response = json_decode($response, true);

        return $response['iamToken'];
    }

    function cevir($deger,$dil){
        $token = $this->yandexToken();


        $curl = curl_init();

        curl_setopt_array($curl, array(
            CURLOPT_URL => 'https://translate.api.cloud.yandex.net/translate/v2/translate',
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'POST',
            CURLOPT_POSTFIELDS =>'{
    "folderId": "b1g1l9b33ueptm8liaou",
    "texts": "'.$deger.'",
    "targetLanguageCode": "'.$dil.'"
}',
            CURLOPT_HTTPHEADER => array(
                'Authorization: Bearer '.$token,
                'Content-Type: application/json'
            ),
        ));

        $response = curl_exec($curl);
        $response = json_decode($response, true);

        return $response['translations'][0]['text'];

    }

    public function ceviri($deger,$suanDil='')
    {
        $session = $this->session;

        $em = $this->em;
        if ($suanDil=='') {
            $suanDil = $session->getSession()->get('dil');
        }else{
            $qb = $em->createQueryBuilder();
            $dill=$qb->select('d.id,d.kisa,d.uzun')
                ->from(Dil::class, 'd')
                ->where('d.id='.$suanDil)
                ->getQuery()
                ->getScalarResult();

            $suanDil = $dill[0];
        }

        $qb = $em->createQueryBuilder();
        $ceviriTara=$qb->select('c.deger,c.id')
            ->from(Ceviri::class, 'c')
            ->join('c.dil','d')
            ->where("c.deger='$deger'")
            ->getQuery()
            ->getScalarResult();

        if (count($ceviriTara) > 0) {
            if ($suanDil['id'] != 1) {
                $qb = $em->createQueryBuilder();
                $ceviriTara2=$qb->select('c.deger')
                    ->from(Ceviri::class, 'c')
                    ->join('c.dil','d')
                    ->where("d.id=".$suanDil['id']." and c.trId=".$ceviriTara[0]['id'])
                    ->getQuery()
                    ->getScalarResult();

                if (count($ceviriTara2) > 0) {
                    $veri = $ceviriTara2[0]['deger'];
                } else {
                    if ($session->getSession()->get('dil')['id'] != '1') {

                        $deger = $this->cevir($deger,$suanDil['kisa']);


                        $dill = $em->getReference(Dil::class,$suanDil['id']);
                        $ceviri = new Ceviri();
                        $ceviri->setDil($dill);
                        $ceviri->setDeger($deger);
                        $ceviri->setTrId($ceviriTara[0]['id']);
                        $em->persist($ceviri);
                        $em->flush();
                        $veri = $deger;
                    }
                }
            } else {
                $veri = $deger;
            }
        }
        else {
            if ($suanDil['id'] == 1) {
                $dill = $em->getReference(Dil::class,$suanDil['id']);
                $ceviri = new Ceviri();
                $ceviri->setDil($dill);
                $ceviri->setDeger($deger);
                $ceviri->setTrId(0);
                $em->persist($ceviri);
                $em->flush();
                $veri = $deger;
            }
            else {
                $dilTr = $em->getReference(Dil::class,1);

                $ceviri = new Ceviri();
                $ceviri->setDil($dilTr);
                $ceviri->setDeger($deger);
                $ceviri->setTrId(0);
                $em->persist($ceviri);
                $em->flush();


                $deger = $this->cevir($deger,$suanDil['kisa']);

                $dill = $em->getReference(Dil::class,$suanDil['id']);
                $ceviriDil = new Ceviri();
                $ceviriDil->setDil($dill);
                $ceviriDil->setDeger($deger);
                $ceviriDil->setTrId($ceviri->getId());
                $em->persist($ceviriDil);


                $em->flush();

                $veri = $deger;

            }

        }






        return $veri;

    }


}